/*** A_ADD_PSTAT.   Add a person to a conference as a member.
/
/    ok = a_add_pstat (pstat, mask, wait);
/
/    Parameters:
/       int       ok;       (return status)
/       Pstat     pstat;    (object containing status info)
/       int4      mask;     (ignored)
/       int       wait;     (synchronicity flag)
/
/    Purpose:
/       Add a person to a conference as a member.  Initialize their
/       "status" record for this conference to "nothing seen".
/
/    How it works:
/       Caller must fill in pstat->cnum, pstat->owner.
/
/    Returns: A_OK on success
/             A_NOCONF    if no such conference
/             A_NOREAD    person not allowed in conference
/             A_NOPERSON  if no such person
/             A_NOTOWNER  owner is not current userid
/             A_NOTNEW    already a member
/
/    Error Conditions:
/
/    Side effects: 
/
/    Related functions:
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:
/       This version uses the global "thisconf", for efficiency(?).
/       Future versions may wish to allocate their own local
/       partic_t.
/
/    Home:  api/agetpsta.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 12/09/92 22:40 New function. */
/*: CR 12/13/92 22:29 Add comments. */
/*: CR  2/22/93 23:31 Add new args to a_cache() calls. */
/*: CP  5/06/92 12:30 Add L(0) as modnames 'item' arg. */
/*: CP  8/01/93  9:37 Add debugging stuff. */
/*: CR  5/15/95 21:15 Clear pstat's # of items seen. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "api.h"
#include "done.h"

extern struct partic_t  thisconf;

FUNCTION  a_add_pstat (Pstat pstat, int4 mask, int wait)
{
   int    success, access, additem, chgresp, ok, chgsubj, error;
   Chix   userid, str;
   Person person;
   Namelist add, del, nlnode();

   if (disk_failure (0, &error, NULL))                    return (A_DBERR);

   /*** Check validity of arguments. */
   if (pstat == (Pstat) NULL  ||  pstat->tag != T_PSTAT)  return (A_BADARG);
   if (pstat->cnum <= 0)                                  return (A_NOCONF);

   /*** Allocate miscellaneous stuff. */
   person = a_mak_person ("aaddpsta person");
   userid = chxalloc (L(20), THIN, "a_add_ps userid");
   str    = chxalloc (L(40), THIN, "a_add_ps str");

   /*** User may only affect their own pstat. */
   if (NOT (mask & P_OVERRIDE)) {
      sysuserid (userid);
      if (NOT chxeq (userid, pstat->owner))               DONE   (A_NOTOWNER);
   }

   /*** Make sure user can access this conference. */
   ok = a_cache (V_GET, pstat->cnum, &access, &additem, &chgresp,
                        &chgsubj, nullchix);
   if (access < 1)                                        DONE   (A_NOREAD);

   /*** Make sure user is not already a member of this conference. */
   if (NOT unit_lock  (XUPA, WRITE, L(pstat->cnum), L(0), L(0), L(0), 
           pstat->owner))                                 DONE   (A_DBERR);
   if (unit_check (XUPA))     { unit_unlk (XUPA);         DONE   (A_NOTNEW); }

   /*** Initialize user's record to "nothing seen". */
   thisconf.items = 0;
   pstat->items   = 0;
   ok = store_user (&thisconf);
   unit_unlk (XUPA);
   if (NOT ok)       DONE (A_NOPERSON);

   /*** Get person's name... */
   chxcpy (person->owner, pstat->owner);
   ok = a_get_person (person, P_NAME | P_LASTNM, A_WAIT);
   if (ok != A_OK)   DONE (ok);

   /*** ... and then add the user to the membership list for this conference. */
   chxformat (person->name, CQ(" (%s)"), L(0), L(0), pstat->owner, nullchix);
   tomember (str, person->lastname, person->name, pstat->owner);
   add = nlnode(1);
   del = nlnode(1);
   nladd  (add, str, 0);
   modnames (XCMD, XCMN, nullchix, pstat->cnum, L(0), add, del, NOPUNCT);
   nlfree (add);
   nlfree (del);

   pstat->ready = 1;
   DONE (A_OK);

done:
   a_fre_person (person);
   chxfree (userid);
   chxfree (str);
   return  (success);
}
