/*** A_GET_CLIST.   Get a list of available conferences.
/
/    ok = a_get_clist (clist, mask, wait);
/
/    Parameters:
/       int       ok;       (return status)
/       Clist     clist;    (object to fill with linked list of conf's)
/       int4      mask;     (mask of properties to get)
/       int       wait;     (synchronicity flag)
/
/    Purpose:
/       Place a linked list of conferences available to the current userid
/       into CLIST.
/
/    How it works:
/       If the LNAME member of the header node of CLIST is empty, then
/       put *all* of the available conferences, one per node, into CLIST.
/
/       Otherwise, LNAME contains a space-separated list of possible
/       conference names.  Put the conferences that (substring) match
/       any name in LNAME into CLIST.
/
/       This code was extensively scavenged from conf/listconf.c.  It scans
/       through the MISC/confsNNN files, and examines each conference,
/       determining if it should add the conference to the end of CLIST.
/
/       A conference only appears in the CLIST if the user can "see"
/       it.  (I.e., the user has joined it, can join it, or the 
/       conference is visible to everyone.)
/
/       CLIST is always emptied at the beginning of a_get_clist().
/
/       The user's membership in the conference is only determined if
/       MASK includes the P_MEMBER bit.  MASK is otherwise ignored.
/
/    Returns: A_OK on success
/             A_DBERR on database error
/
/    Error Conditions:
/  
/    Side effects: 
/
/    Related functions:
/
/    Called by:  UI
/
/    Operating system dependencies:
/
/    Known bugs:
/       This version does not fill in the "true" conference name field
/       in a Clist.
/
/    Home:  api/agetclis.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR 11/15/92 17:47 New function. */
/*: CR 11/17/92 18:28 Enforce XCTC file for each conference. */
/*: CR 11/27/92 19:20 Better arg checking. */
/*: CL 12/11/92 11:12 Long Chxcatsub args. */
/*: CL 12/11/92 13:46 Long Chxvalue N. */
/*: CR  1/11/93 22:07 Add to comments. */
/*: CP  8/01/93  9:37 Add debugging stuff. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */
/*: JV  9/26/93 13:08 Add ENTRY to match RETURN. */
/*: CR  4/11/95 18:11 Add LNAME matching feature; P_MEMBER use. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "api.h"
#include "unitwipe.h"
#include "done.h"

extern int             debug;
extern Chix            confid;
extern short           confnum;
extern union  null_t   null;

FUNCTION  a_get_clist (Clist clist, int4 mask, int wait)
{
   struct  namlist_t *namdir, *this, *groups, *nlnode();
   int     member, savecnum, success, access, matchall;
   short   i, ok;
   Chix    name, code, userid, match;
   int4    cnum, c1, pos;
   Textbuf tbuf, make_textbuf();
   Clist   thisconf, end_clist, a_mak_clist();

   ENTRY ("a_get_clist", "");

   if (clist == (Clist) NULL  ||  clist->tag != T_CLIST)  RETURN (A_BADARG);

   /*** Allocate and initialize working chix. */
   name     = chxalloc (L(80),  THIN, "a_get_cl name");
   code     = chxalloc (L(20),  THIN, "a_get_cl code");
   userid   = chxalloc (L(16),  THIN, "a_get_cl userid");
   sysuserid (userid);

   /*** Get the list of conf's to match. */
   match    = chxalloc (L(80),  THIN, "a_get_cl match");
   chxcpy    (match, clist->lname);   /* List of conf's to match? */
   jixreduce (match);
   matchall = (chxvalue (match, L(0)) == 0);

   /*** Empty CLIST, in case we were passed an old one by the caller. */
   if (clist->next != (Clist) NULL)   a_fre_clist (clist->next);
   clist->next  = (Clist) NULL;
   end_clist    = clist;
   clist->ready = 1;

   /*** Allocate a textbuf to contain the confsNNN partfiles. */
   tbuf = make_textbuf (TB_RESPS, TB_LINES, "agetclis tbuf");

   /*** Load the Conference names directory. */
   if (NOT unit_lock (XSCD, READ, L(0), L(0), L(0), L(0), nullchix))  DONE (A_DBERR);

   namdir = nlnode(4);
   ok = loadnmdir (XSCD, namdir);
   unit_unlk (XSCD);
   if (NOT ok)  { nlfree (namdir);               DONE (A_DBERR); }

   /*** Load each Conference namepart file in turn, and if the user is
   /    appropriately permitted, add the conference to CLIST. */
   unit_wipe  (LCNF);
   groups = nlnode (1);
   savecnum = confnum;
   for (this=namdir->next;   this != null.nl;   this = this->next) {
      if (this->num <= 0)  continue;
      if (NOT unit_lock(XSCN, READ, L(0), L(this->num), L(0), L(0), nullchix)) break;
      if (NOT loadnpart(XSCN, this->num, tbuf))    { unit_unlk (XSCN);   break;}
      unit_unlk (XSCN);
   
      /*** For each line in the conference name part file 
      /    (i.e, for each conference)... */
      for (i=0;   i<tbuf->l_used;   ++i) {
         chxtoken (name, nullchix, 1, tbuf->line[i]);
         chxtoken (code, nullchix, 2, tbuf->line[i]);
         c1 = chxvalue (name, L(0));
         if (c1 == 0  ||  c1 == '!')                            continue;
 
         /*** Only check Local Confnames, but allow old syntax (6=='\0') */
         if (chxvalue (code, L(6)) == 'T')                        continue;
      
         /*** Do we add this conf to the resulting list? */
         if (NOT matchall  &&  tablepart  (match, name) == -1)  continue;

         /*** Determine the conference number. */
         pos  = 2;
         cnum = chxint4 (code, &pos);    if (pos == L(2))         continue;
         confnum = cnum;

         /*** Earlier versions of a_get_clist() checked for an XCTC
         /    file, and created one if not found.  This has been
         /    removed.  Is it possible we might have to put it back? */

         /*** Determine if the user is already a member. */
         member = 0;
         if (mask & P_MEMBER) {
            if (unit_lock (XUPA, READ, L(cnum), L(0), L(0), L(0), userid)) {
               member = unit_check (XUPA);
               unit_unlk (XUPA);
            }
         }

         /*** Determine this user's access to the conference. */
         allowed_on (nullchix, null.nl, -1);  /* Flush cache. */
         access = allowed_on (userid, groups, confnum);
         unit_wipe (LCNF);

         /*** If the user did/can-not join, and the conference is invisible,
         /    then skip it. */
         if (NOT member  &&  access==0  &&  chxvalue (code, L(0)) == 'c'  &&
             NOT (mask & P_OVERRIDE) )      continue;
            
         /*** Add conference to list. */
         thisconf = a_mak_clist("agetclis thisconf");
         chxcpy (thisconf->lname, name);
         thisconf->cnum    = confnum;
         thisconf->member  = member;
         thisconf->access  = access;
         thisconf->visible = (chxvalue (code, L(0)) == 'o');

         /*** Add THISCONF onto the end of CLIST. */
         end_clist->next = thisconf;
         end_clist       = thisconf;
      }
   }

   nlfree (namdir);
   nlfree (groups);
   confnum = savecnum;
   DONE (A_OK);

done:
   free_textbuf (tbuf);
   chxfree (name);
   chxfree (code);
   chxfree (userid);
   chxfree (match);

   RETURN  (success);
}
