
/*** CHXERROR.  Handle error conditions from Chix package functions.
/
/    chxerror (error, s1, s2)
/
/    Parameters:
/       int   error;    ( Chix error code number CXE... )
/       char *s1;       ( Name of calling function. )
/       char *s2;       ( Other information (chix name, etc.) )
/
/    Purpose:
/       Handle error conditions that occur inside chix functions.
/       Since the chix package can (in theory) be used in a variety
/       of applications, any application-specific error handling
/       should be enclosed in #if/#endif's in chxerror().
/
/    How it works:
/       chxerror() is called from inside other chix functions when they
/       detect an error condition.  The ERROR code describes the kind
/       of error condition, such as:
/          CXE_NULL    chix argument is a null pointer
/          CXE_FREED   chix argument is a freed chix
/          CXE_NOTCHX  chix argument is not a chix at all
/          CXE_NOMEM   insufficient heap memory to allocate a chix
/
/       chxerror() displays an appropriate error message (based on ERROR),
/       and takes appropriate action.  (Sometimes this means just 
/       displaying a message.  More serious errors may mean exiting
/       immediately.)
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: chxcheck(), any many other chix functions.
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  chx/chxerror.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: JV 10/16/92 16:11 Add CXE_SELF, CXE_UNKNOWN. */

#include <stdio.h>
#include "chixdef.h"

#define  CAUCUS   0     /* Chix used in Caucus application? */

#if CAUCUS
#include "caucus.h"
extern char hangup;
#endif

FUNCTION  chxerror (error, s1, s2)
   int    error;
   char  *s1, *s2;
{
   ENTRY ("chxerror", "");

   switch (error) {

      case (CXE_NORESV):
         printf ("CXE_NORESV (%d), cannot allocate reserve memory in '%s'.\n",
                  error, s1);
#if CAUCUS
         sysexit (EXIT_CHX_NORESV);
#endif
         break;

      case (CXE_LOMEM):
         printf ("CXE_LOMEM (%d), %s '%s'.\n  String='%s'\n",
                  error, "running out of heap memory in", s1, s2);
#if CAUCUS
         hangup = 1;
#endif
         break;

      case (CXE_NOMEM):
         printf ("CXE_NOMEM (%d), all out of heap memory in '%s'.\n",
                  error, s1);
#if CAUCUS
         sysexit (EXIT_CHX_NOMEM);
#endif
         break;


      case (CXE_NULL):
         printf ("CXE_NULL (%d), chix argument is null pointer in '%s'.\n",
                  error, s1);
         break;

      case (CXE_FREED):
         printf ("CXE_FREED (%d), chix argument in '%s' was freed!\n",
                  error, s1);
         printf ("  Old name of chix was '%s'\n", s2);
         break;

      case (CXE_NOTCHX):
         printf ("CXE_NOTCHX (%d), argument not a chix in '%s'.\n",
                  error, s1);
         break;

      case (CXE_RESIZE):
         printf ("CXE_RESIZE (%d), chix '%s' resized too often.\n",
                  error, s2);
         break;

      case (CXE_NOEND):
         printf ("CXE_NOEND (%d), chix '%s' had no 0 terminator!\n",
                  error, s2);
         break;

      case (CXE_SELF):
         printf ("CXE_SELF (%d), used chix '%s' twice in function call.\n",
                  error, s2);
                   
      default:
         printf ("CXE_UNKNOWN (%d), unknown chix error for chix '%s'.\n",
                  error, s2);
         break;
   }

#if CAUCUS
/*    hangup = 1; */
#endif

   RETURN (1);
}
