
/*** CHXMEM.  Allocate a block of memory for chix internal functions.
/
/    p = chxmem (size, name);
/
/    Parameters:
/       char *p;
/       int4  size;        (requested size of memory block)
/       char *name;        (name associated with block by caller)
/
/    Purpose:
/       Allocate a block of SIZE bytes for use in a chix, and return a
/       pointer to that memory.  NAME is a regular C character string
/       containing the name of a chix string, used for debugging and
/       error handling.
/
/    How it works:
/       Chxmem just calls calloc() to allocate memory off the heap.
/       It does some intelligent(?) error-handling, see below.
/
/    Returns:
/       A (char *) pointer to the allocated memory.
/
/    Error Conditions:
/       On the first call to chxmem(), it allocates (internally) a block
/       of RESERVE_SIZE bytes.  If chxmem() ever fails to allocate the
/       memory it needs, it frees the "reserve" and tries again.  If
/       the second attempt succeeds, chxmem() displays a diagnostic,
/       and tries to allocate the requested memory.
/
/       Once the reserve has been freed, if any memory allocation fails,
/       chxmem() displays an error message and exits via chxerror().
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  chxalloc(), chxresize().
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  chx/chxmem.c.
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR 12/11/92 12:46 Chxmem 1st arg is int4. */
/*: CR  9/18/97 13:43 Replace sysmem with calloc(). */

#include <stdio.h>
#include "chixdef.h"

#define  RESERVE_SIZE   L(4096)

static char  first_call = 1;   /* First call to chxmem()? */
static char *reserved;         /* Block of memory reserved for emergency. */

FUNCTION  char *chxmem (size, strname)
   int4   size;
   char  *strname;
{
   char  *p;
   char  *calloc();

   ENTRY ("chxmem", "");

   /*** If first call to chxmem(), reserve a block of memory
   /    for memory failure emergency. */
   if (first_call) {
      first_call = 0;
      if ( (reserved = calloc (1, (unsigned) RESERVE_SIZE)) == NULL) 
         chxerror (CXE_NORESV, "chxmem", "");
   }

   if ( (p = calloc (1, (unsigned) size)) == NULL) {
      free (reserved);
      reserved = NULL;
      chxerror (CXE_LOMEM, "chxmem", strname);

      if ( (p = calloc (1, (unsigned) size)) == NULL)  
         chxerror (CXE_NOMEM, "chxmem", "");
   }

   RETURN (p);
}
