
/*** DEL_RESPONSE.  Delete a set of (items and) responses.
/
/    ok = del_response (range);
/
/    Parameters:
/       int    ok;          (success?)
/       Chix   range;       (range of items/resp's selected on command line)
/
/    Purpose:
/       Caucus CLI: handle user's request to delete one or more items 
/       or responses in a given range.
/
/    How it works:
/       Calls parse_or_ask() to parse RANGE into an Rlist.  Displays a
/       single line per Rlist entry, summarizing the item/resps to be
/       deleted, then asks the user for confirmation to delete them all.
/
/       Then, for each entry in the Rlist:
/          (a) If the RESPONSES modifier was not found, *and* the Rlist entry
/              includes response 0, delete the entire item.
/
/          (b) Otherwise, delete all the responses in the entry.
/       
/    Returns: 1 on success
/             0 if user cannot delete *any* items
/             0 if no RANGE was specified
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions: 
/
/    Called by:  del()
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  del/delresp.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved.  */

/*: CP  6/30/93 17:03 New function. */
/*: CP  7/16/93 17:17 Add 'attach' arg to lead_header() call. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */
/*: CP  8/06/93 15:06 Use r_bound() to limit first/last responses. */
/*: CP  8/09/93 11:41 Add 'attach' arg to new_parse() call. */
/*: CP  8/27/93 17:26 Add DEL ATTACH handling. */
/*: CP  9/09/93 17:11 Replace new_parse() with parse_or_ask(). */
/*: CK  9/13/93 13:04 Use r_bound() for both loops! */
/*: CK  9/15/93 14:04 Add 'mod1st' arg to parse_or_ask(). */
/*: CK 10/07/93 17:05 Use &thisitem in parse_or_ask() call. */
/*: CK 10/14/93 14:57 mod[M_ITEMS] -> NOT mod[M_RESPONSES]. */
/*: CR 10/18/93 13:30 Add comments re Lint. */
/*: CR 10/22/93 13:49 Pass 'first' instead of ptr->r0 to show_resp. */
/*: CP 11/11/93 13:37 Add 'resp1' arg to show_format() call. */
/*: CP 11/18/93 15:35 Add cnum, inum args to r_bound() call. */
/*: CP 11/28/93 18:42 Use 'first' as resp # in show_format() call. */
/*: CP 12/10/93 21:31 Add 'final' arg to r_bound() call. */

#include <stdio.h>
#include "caucus.h"
#include "modifier.h"
#include "api.h"
#include "xaction.h"
#define   RLNULL   ( (Rlist) NULL )

extern struct master_template master;
extern struct this_template   thisitem;
extern Userreg                thisuser;
extern Textbuf                tbuf;
extern union  null_t          null;
extern Chix                   yesnotable;
extern short                  confnum;

FUNCTION  del_response (range)
   Chix   range;
{
   Chix   title, header, itemform, respform, def, author, answer;
   int    i, am_author, final;
   char   modifier[MODIFIERS+5];
   char  *Pptr, *Hptr;
   int    success, first, last, found;
   int    status;    /* purely for debugging, Lint will complain. */
   Rlist  result, rptr;
   Resp   resp;

   ENTRY ("del_resp", "");

   if (thisuser->read_only) {
      mdwrite (XWER, "ent_Ereadonly", null.md);
      RETURN  (0);
   }

   if (NOT thisuser->is_organizer  &&  master.nochange) {
      mdwrite (XWER, "chg_Tnochgr", null.md);
      RETURN  (0);
   }

   def      = chxalloc (L(20), THIN, "del_resp def");
   title    = chxalloc (L(80), THIN, "del_resp title");
   header   = chxalloc (L(80), THIN, "del_resp header");
   author   = chxalloc (L(80), THIN, "del_resp author");
   itemform = chxalloc (L(80), THIN, "del_resp itemform");
   respform = chxalloc (L(80), THIN, "del_resp respform");
   answer   = chxalloc (L(80), THIN, "del_resp answer");
   result   = RLNULL;
   resp     = a_mak_resp("delresp resp");
   success  = 0;

   mdstr (itemform, "res_Fshowhead3", null.md);
   mdstr (respform, "res_Fshowhead4", null.md);

   /*** Parse the RANGE... */
   if (NOT parse_or_ask (range, &result, modifier, nullchix, confnum, 
                                &thisitem, 1))  FAIL;

   /*** Display the brief listing of the things to be deleted. */
   sysbrkclear();
   found = 0;
   for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
      for (i=rptr->i0;   i<=rptr->i1;   ++i) {
         if (i == 0)                                               continue;
         if (NOT r_bound (&first, &last, &final, rptr, confnum, i, 
                                            master.responses[i]))  continue;
         if (sysbrktest())  break;

         if (NOT load_header (confnum, i, header, title, (Attachment) NULL))
                                             continue;
         if (EMPTYCHX(header))               continue;

         if (NOT modifier[M_RESPONSES]  &&  first == 0  &&  
             NOT modifier[M_ATTACH]) {
            chxtoken (author, null.chx, 2, header);
            am_author = equal_id (author, thisuser->id, nullchix);
            if (NOT am_author  &&  NOT thisuser->is_organizer)   continue;
            found = 1;
            show_format (XWTX, itemform, i, 0, 0, 0, 0, title, header);
         }
         else {
            found = 1;
            show_format (XWTX, respform, i, first, first, last, 0, title, header);
         }
      }
   }

   /*** If no items/resps were found, tell user and go home. */
   if (NOT found) {
      mdwrite (XWER, "del_Tnysel", null.md);
      FAIL;
   }

   if (NOT thisuser->is_organizer)  mdwrite (XWTX, "del_Tnotyours", null.md);

   /*** Confirm deletion of the remaining items. */
   chxtoken (def, null.chx, 1, yesnotable);
   if (modifier[M_ATTACH])   { Pptr = "del_PattaD";   Hptr = "del_HattaD"; }
   else                      { Pptr = "del_PitemD";   Hptr = "del_HitemD"; }
   if (mdprompter (Pptr, yesnotable, 2, Hptr, 10, answer, def)
          < 1)  SUCCEED;

   /*** Actually do the deletion. */
   sysbrkclear();
   resp->cnum = confnum;
   for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
      if (sysbrktest())     break;
      for (i=rptr->i0;   i<=rptr->i1;   ++i) {
         if (sysbrktest())  break;
         if (NOT r_bound (&first, &last, &final, rptr, confnum, i, 
                                         master.responses[i]))  continue;
         resp->inum = i;

         /*** Delete entire ITEM. */
         if (NOT modifier[M_RESPONSES]  &&  NOT modifier[M_ATTACH] 
                                        &&  rptr->r0 == 0) {
            resp->rnum = 0;
            status = a_del_resp (resp, P_EVERY, A_WAIT);
            continue;
         }

         /*** Otherwise, delete responses... */
         for (resp->rnum=first;   resp->rnum<=last;   ++resp->rnum) {
            status = a_del_resp (resp, (modifier[M_ATTACH] ? P_ATTACH : P_EVERY),
                                        A_WAIT);
         }
      }
   }
   SUCCEED;

done:
   a_fre_resp  (resp);
   a_fre_rlist (result);
   chxfree (def);
   chxfree (title);
   chxfree (author);
   chxfree (header);
   chxfree (itemform);
   chxfree (respform);
   chxfree (answer);
   RETURN  (success);
}
