/*** DOECHO.    Echo the arguments the user typed.  Mostly useful in macros.
/
/    doecho (command);
/
/    Parameters:
/      Chix command;   (The command line.)
/
/    Purpose:
/      Caucus ECHO command normally echos arguments to screen.  It also
/      parses and understands "<" (display contents of file instead of
/      echoing command line), ">" (echo results to a file), and ">>"
/      (append to a file).
/
/    How it works:
/      ECHO understands several "dash" options, described below:
/        -n    do not follow echoed text with default newline.
/        -s    echo dictionary string named after -s.
/        -i    echo if even sysbreak interrupt already detected.
/        -c    clear screen, ignore rest of ECHO command.
/        -p    pause for user to press RETURN
/        -h    display text in help window (VM/CMS only)
/        -f    like -p, but whole ECHO command ignored if fullscreen off.
/        -q    (requires -p) text & RETURN typed by user not echoed.
/        -u    "unfiltered", no word-wrapping or screen-size pausing.
/
/    Returns:
/       1 in all cases except:
/       0 if request to display text in HELP window & flag.fullscreen not set.
/   
/    Error Conditions:
/   
/    Side Effects:
/ 
/    Related to:
/   
/    Called by: interpret()
/   
/    Known bugs:
/       "-u" only works up to 32767 characters worth (after which it
/       "word wraps"?!).
/
/    Home: cmd2/doecho.c
/ */

/*: AA  7/29/88 17:39 Source code master starting point */
/*: CW  6/07/89 14:09 Change name to doecho() to avoid curses conflict. */
/*: CR  9/07/89 20:37 Add handling for -s, -n ECHO options. */
/*: CR 10/20/89 14:26 Add handling for -i (no interrupt) option. */
/*: CR 10/25/89 15:16 Add -c (clear screen) and -p (pause) options. */
/*: CR 11/21/89 18:01 Prompts with pause go to prompt window. */
/*: CR 11/22/89 17:40 Rearrange position of clear screen function. */
/*: JV  2/23/90 06:00 Add echo_offs to sysread(). */
/*: CR  3/01/90 12:13 Add -h (output to help window) option.  */
/*: CR  3/21/90 17:04 Add 4th arg to sysread(). */
/*: CR  3/24/90 14:25 Call sysread (0... for echo -pause. */
/*: CR  4/10/90 14:53 Remove echo_offs. */
/*: CR  6/05/90 21:35 "-f" => -p, unless not fullscreen, which cancels all! */
/*: CR  7/02/90  7:25 Adjust output unit *after* parse_output() call. */
/*: CR  7/17/90 12:42 Remove extra unit_unlk(in). */
/*: CJ 11/22/90 15:35 Add -q "quiet" option. */
/*: CJ 11/22/90 22:35 Use new form of systurn() call. */
/*: CJ 11/26/90 17:10 Add new OS_ECHO systurn() stuff. */
/*: JX  2/22/92 15:00 Chixify and add header. */
/*: JX  5/26/92 19:44 Fixify. */
/*: CX  6/05/92 15:32 Replace sysread() with unit_read(). */
/*: CX  6/07/92 19:46 Add 3rd arg to unit_read() call. */
/*: JX  6/09/92 17:25 chxtoken returns -1 on fail. */
/*: CR  6/09/92 14:19 Add 2nd arg to parse_input. */
/*: CR  8/14/92 11:24 Always pause on -p, regardless of com_unit. */
/*: CI 10/06/92 16:59 Chix Integration. */
/*: TP 10/19/92 15:30 Replace unit_read() with input_control(). */
/*: TP 10/20/92 12:28 Pluck output redirect out of REST. */
/*: CR 10/30/92 16:23 Dont muck with terminal if FILEOUT. */
/*: CL 12/10/92 17:06 Long ascofchx args. */
/*: CP  8/10/93 13:37 Use new Redirect object in handling redirection. */
/*: CK  8/20/93 14:57 Add -u option. */
/*: CP  9/03/93  0:27 Check ired.locked before reading from ired.unit. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "chixuse.h"
#include "done.h"
#include "units.h"
#include "redirect.h"

extern struct screen_template used, screen;
extern struct termstate_t     term_in_caucus;
extern struct flag_template   flag;
extern union  null_t          null;

FUNCTION  doecho (command)
   Chix   command;
{
   static   Chix echotab;
   struct   screen_template  save;
   int      option, i, newline, dicstring, clear, pause;
   int      noint, usehelp, quiet, success, fileout;
   Chix     word, rest, line;
   char     wordchar[100];
   Redirect ired, ored;

   ENTRY ("echo", "");

   word = chxalloc (L(100), THIN, "doecho word");
   rest = chxalloc (L(100), THIN, "doecho rest");
   line = chxalloc (L(100), THIN, "doecho line");

   mdtable ("ech_Aopts", &echotab);

   /*** Parse and apply redirection information.  If input redirection
   /    has occurred, output redirection is ignored. */
   parse_rin  (&ired, command, XIRE);
   parse_rout (&ored, (ired.unit==XIRE ? /*empty*/ line : command));
   apply_redirect (&ired);
   apply_redirect (&ored);

   /*** Parse the ECHO command line for i/o redirection and echo
   /    options (e.g., -n, -s, -i). */
   save = screen;
   newline   = 1;
   dicstring = clear = pause = noint = usehelp = quiet = 0;
   for (i=2;   (1);   ++i) {
      if ( (chxtoken (word, rest, i, command)) == -1)   break;
      if ( (option = tablematch (echotab, word)) < 0)   break;
      switch (option) {
         case (0):   newline   = 0;            break;   /* -n */
         case (1):   dicstring = 1;            break;   /* -s */
         case (2):   noint     = 1;            break;   /* -i */
         case (3):   clear     = 1;            break;   /* -c */
         case (4):   pause     = 1;            break;   /* -p */
         case (5):   usehelp   = 1;            break;   /* -h */

         case (6):   if (NOT flag.fullscreen)  FAIL;
                     pause     = 1;            break;   /* -f */

         case (7):   quiet     = 1;            break;   /* -q */

         case (8):                                      /* -u */
            screen.width = 32767;
            screen.lines = 0;
            used.width   = 0;
            break;
      }
   }

   /*** The output unit gets modified under certain conditions:
   /    -i => no <CANCEL> interruption;
   /    -h => output goes to help window;
   /    -p => pause (no \n) prompts got to prompt window. */
   if (noint    &&   ored.unit==XWTX)                      ored.unit = XWTY;
   if (usehelp  &&  (ored.unit==XWTX || ored.unit==XWTY))  ored.unit = XWHL;
   if (pause    &&  (ored.unit==XWTX || ored.unit==XWTY))  ored.unit = XWPM;

   /*** Perform the appropriate form of the ECHO command: dictionary
   /    string, text following ECHO command, or text from <file. */
   fileout = (unit_control (XWTX, UNIT_GETSUB) != XWTX);
   if      (clear) { 
      if (NOT fileout) sysclrscn();
      used.lines = used.width = 0;
   }

   else if (ired.unit == XKEY  &&  dicstring) {
      ascofchx (wordchar, word, L(0), L(98));
      mdwrite  (ored.unit, wordchar, null.md);
   }

   else if (ired.unit == XKEY) {
      unit_write (ored.unit, rest);
      if (newline)    unit_write (ored.unit, CQ("\n"));
   }

   else if (ired.locked  &&  unit_view (ired.unit)) {
      sysbrkclear();
      while (unit_read (ired.unit, line, L(0))) {
         unit_write (ored.unit, line);
         unit_write (ored.unit, CQ("\n"));
         if (ored.unit==XWTX  &&  sysbrktest())  break;
      }
      unit_close (ired.unit);
   }

   if (ired.locked)   unit_unlk  (ired.unit);
   if (ored.locked) { unit_close (ored.unit);    unit_unlk  (ored.unit); }

   if (pause) {
      if (quiet)  systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
      sysbrkclear();
      sysflushin ();
      input_control (IC_GETLIN | IC_SESS, word);
      if (quiet) {
         if (NOT fileout)  sysputstr ("\r");
         systurn   (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, ON);
      }
   }
   SUCCEED;

done:
   screen = save;
   chxfree (word);
   chxfree (rest);
   chxfree (line);
   RETURN  (success);
}
