/*** STATUS_FORMAT.   Display status information according to format string.
/
/    status_format (out, dicstr);
/
/    Parameters:
/       int    out;        (unit number for output)
/       Chix   dicstr;     (name of dictionary string for format)
/
/    Purpose:
/       Display (on unit OUT) certain status information according to a 
/       "tailorable format", selected by dictionary string named DICSTR.
/
/    How it works:
/       Loads dictionary string named DICSTR.  Parses the string and
/       displays the text on unit OUT, interpreting the following
/       directives as shown:
/          &a      this user's name
/          &c      this conference's name
/          &d      today's date
/          &k      name of cancel key
/          &o      the name of the organizer
/          &t      the current time
/          &T      elapsed time (in minutes) since Caucus started
/          &u      this user's userid
/          &I      total number of new items in this conference
/          &M      total number of new messages
/          &N      total number of items with any new responses.
/          &R      total number of new responses in this conference
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  interpret()
/
/    Operating system dependencies:
/
/    Known bugs:
/       Nowhere do we handle number of unseen items.
/
/    Home:  cmd2/statusfo.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved.  */

/*: CR  7/09/92 17:34 New function. */
/*: CR  7/19/92 16:29 Declare systime(). */
/*: CR  8/25/92  4:36 Add &T field. */
/*: CR 10/07/92 23:07 Chixify. */
/*: CR 10/11/92 22:27 Remove CQ from buildline() calls. */
/*: CR 10/13/92 11:35 Add missing break's to case statements. */
/*: CR 12/10/92 15:29 chxindex 2nd arg is int4. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CP 11/30/93 12:49 Add '&L' for last item in conf. */
/*: CR  7/12/94 14:27 Load organizer info only if &o is used. */

#include <stdio.h>
#include "caucus.h"

extern struct master_template master;
extern Userreg                thisuser;
extern union  null_t          null;
extern Chix                   confname;

FUNCTION   status_format (out, dicstr)
   int     out;
   Chix    dicstr;
{
   int     newitems, newresp, actitems, newmess, elapsed_sec;
   int4    v, length;
   int4    pos, t;
   Userreg org, make_userreg();
   char    datestr[80];
   Chix    fstr, date, time, temp, how_s, how_d, authorname;
   Chix    syscancel();
   int4    systime();

   ENTRY ("status_format", "");

   fstr  = chxalloc (L(1000), THIN, "status_format fstr");
   date  = chxalloc (  L(20), THIN, "status_format date");
   time  = chxalloc (  L(20), THIN, "status_format time");
   temp  = chxalloc (  L(20), THIN, "status_format temp");
   how_s = chxalloc (  L(20), THIN, "status_format how_s");
   how_d = chxalloc (  L(20), THIN, "status_format how_d");
   authorname = chxalloc (L(80), THIN, "status_format authorname");

   sysdaytime (date, time, 0, systime());

   removeid (authorname, thisuser->name, thisuser->id, fstr);

   org = make_userreg();

   calcstatus (&newitems, &newresp, &actitems, &newmess);

   /*** Loop through the format string, processing all "&x" directives,
   /    and displaying all other text as is.  */
   mdstr (fstr, ascquick(dicstr), null.md);
   buildline (out, null.chx, L(0), L(-1), BLINIT);
   for (pos=0;   v = chxvalue (fstr, L(pos));   ) {

      /*** If no more "&"s left, display to the end of the string. */
      if ( (t = chxindex (fstr, pos, CQ("&"))) < 0) 
         { buildline (out, fstr, pos, ALLCHARS, BLADD);   break; }

      /*** Treat escaped "&"s as ordinary text to be displayed. */
      else if (chxvalue (fstr, L(t+1)) == '&') {
         buildline (out, fstr, pos, t, BLADD);
         pos = t + 2;
         continue;
      }

      /*** Remaining code in loop processes "&x" forms.   Start by 
      /    displaying all text before the "&". */
      buildline (out, fstr, pos, t-1, BLADD);

      /*** Parse out the string length, including "-" => left-justify, and
      /    make a sprintf format string to use to display the info. */
      pos = t + 1;
      length = chxint4 (fstr, &pos);
      if (pos == t+1) {
         length = 0;
         chxcpy    (how_s, CQ("%s"));
         chxcpy    (how_d, CQ("%d"));
      }
      else {
         chxclear  (how_s);
         chxclear  (how_d);
         chxformat (how_s, CQ("%%%ds"), length, L(0), null.chx, null.chx);
         chxformat (how_d, CQ("%%%dd"), length, L(0), null.chx, null.chx);
      }

      /*** Decide what to display according to "x" code in "&x". */
      chxclear (temp);
      v = chxvalue (fstr, L(pos));
      switch (v) {

         case ('a'):    /* User. */
            chxformat (temp, how_s, L(0), L(0), authorname, null.chx);   break;

         case ('c'):    /* Conf.   */
            chxformat (temp, how_s, L(0), L(0), confname, null.chx);
            chxupper  (temp);
            break;

         case ('d'):    /* Date.   */
            chxformat (temp, how_s, L(0), L(0), date, null.chx);   break;

         case ('L'): 
            chxformat (temp, how_d, L(master.items), L(0), null.chx, null.chx);
            break;

         case ('t'):    /* Time.   */
            chxformat (temp, how_s, L(0), L(0), time, null.chx);   break;

         case ('T'):
            elapsed_sec = elapsed();
            sprintf   (datestr, "%d:%02d", elapsed_sec/60, elapsed_sec % 60);
            chxformat (temp, how_s, L(0), L(0), CQ(datestr), null.chx);   break;
            break;

         case ('u'):   /* (userid). */
            chxformat (temp, how_s, L(0), L(0), thisuser->id, null.chx);   break;

         case ('k'):  /* can key. */
            chxformat (temp, how_s, L(0), L(0), syscancel(), null.chx);    break;

         case ('o'):    /* orga. */
            if (EMPTYCHX(org->name)) 
               loadreg  (master.organizerid, org, NULLSITE);
            chxformat (temp, how_s, L(0), L(0), org->name, null.chx);      break;

         case ('I'): 
            chxformat (temp, how_d, L(newitems), L(0), null.chx, null.chx);
            break;

         case ('R'):
            chxformat (temp, how_d, L(newresp), L(0), null.chx, null.chx);
            break;

         case ('N'):
            chxformat (temp, how_d, L(actitems), L(0), null.chx, null.chx);
            break;

         case ('M'):
            chxformat (temp, how_d, L(newmess), L(0), null.chx, null.chx);
            break;

         case ('z'):                                            /* Tab pos. */
            buildline (out, temp, L(0), L(0), (int) length);
            break;

         default:    break;
      }

      /*** Enforce maximum length of info, and display it. */
      if (sysbrktest())  break;
      buildline (out, temp, L(0), (length>0 ? length-1 : ALLCHARS), BLADD);
      ++pos;
   }
   buildline (out, null.chx, L(0), L(-1), BLFLUSH);

   free_userreg (org);
   chxfree (date);
   chxfree (time);
   chxfree (fstr);
   chxfree (temp);
   chxfree (how_s);
   chxfree (how_d);
   chxfree (authorname);
   RETURN  (1);
}
