/*** ALLOWED_ON.  Is a user ALLOWED_ON this conference?
/
/    ok = allowed_on (userid, groups, cnum);
/
/    Parameters:
/       int      ok;        (Success?)
/       Chix     userid;    (userid or "@host" host name)
/       Namelist groups;    (history of previous matches against GROUP files)
/       int      cnum;      (conference number)
/
/    Purpose:
/       Determine if a userid (or host) is allowed to join (subscribe to)
/       a conference, and if so, at what security level?
/
/    How it works:
/       Allowed_on() examines the XCUS file for conference CNUM to see
/       if USERID is included or excluded from the list of valid users.
/
/       If the XCUS file "<" includes any groups files, the history
/       of matches found in each groups file is recorded (and re-used,
/       where appropriate) in the namelist GROUPS.
/
/       Allowed_on() uses a caching mechanism that must be flushed under
/       certain circumstances.  To flush its cache, call allowed_on() with
/       CNUM = -1.
/
/       Two examples of places the cache must be flushed:
/       1) After the userlist is changed (cus_user())
/       2) The user is about to join a conf (caucus())
/
/       The static CONFNUM is initialized to -1.  Thereafter it is either
/       1) =-1: ULIST does not point to an allocated Chix;
/       2) > 0: ULIST *does* point  to  an allocated Chix;
/
/    Returns: 3 if USERID has organizer privileges
/             2 if USERID may join as a regular member
/             1 if USERID may join as a read-only member
/             0 if USERID may not join
/            -1 if conference CNUM does not exist
/            -2 on any other error
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  XFU, Caucus
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  main/allowedo.c
/ */

/*** Copyright (C) 1985-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR  4/13/89 11:47 Take wildcard character from dictionary. */
/*: CW  5/18/89 14:53 Add XW unit codes. */
/*: CR  9/15/89 16:46 Add ":readonly" class of users. */
/*: CR  7/13/90 14:10 Add ":organizer" class. */
/*: CR  7/17/90 11:46 Add group matching, via groupmatch() call. */
/*: CR  9/12/90 13:39 Fix interaction between :readonly & :organizer. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/25/91  1:09 Trim trailing blanks from entries in XCUS! */
/*: CR  6/20/91 16:19 Use equal_id() to parse host information. */
/*: CR  7/24/91 21:45 Remove unused variables. */
/*: CR 10/21/91 18:15 Add 3rd arg, change 1st arg & return codes. */
/*: CR 10/30/91 17:32 Use ulist's. */
/*: JX  3/04/92 11:20 Chixify. */
/*: DE  3/26/92 16:37 Patched chxindex w/ 0, */
/*: DE  5/26/92 15:19 Fix chixification */
/*: JX  8/10/92 10:54 Add cache. */
/*: JX 10/01/92 13:45 Protect upper-case for group file names. */
/*: CR 12/02/92 14:34 Use ()'s for confnum=-1. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CR 10/08/97 17:24 Use simplified Ulist. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

#define FLUSH -1

FUNCTION  allowed_on (userid, groups, cnum)
   Chix   userid;
   struct namlist_t  *groups;
   int                cnum;
{
   static int   confnum = (-1);
   static Chix  inextab;
   static Chix  ulist = nullchix;
   static int   permission[] = {0, 2, 0, 1, 3};
   Chix   wildcard, name, line, group_file;
   int    errors, index, type, allowed, match, success;
   int4   pos, c1;

   ENTRY ("allowed_on", "");

   line     = chxalloc (L(100), THIN, "allowed line");
   name     = chxalloc ( L(80), THIN, "allowed name");
   wildcard = chxalloc ( L(20), THIN, "allowed wildcard");
   group_file = chxalloc (L(40),  THIN, "allowed group_file");

   if (ulist == nullchix)  ulist = chxalloc (L(400), THIN, "allowed ulist");

   if (cnum == FLUSH) {                     /* Flush cache. */
      chxclear (ulist);
      confnum = -1;
      SUCCEED;
   }

   /*** Allocate a ULIST to hold the XCUS userlist data. */
   if (confnum == -1)   chxclear (ulist);

   /*** Load the ULIST from the XCUS file. */
   if (confnum != cnum) {
      chxclear (ulist);
      if (NOT load_ulist (XCUS, cnum, ulist, name, &errors)) 
         DONE (-1);
   }

   /*** Primary organizer always has full permissions. */
   if (equal_id (userid, name, nullchix))  DONE (3);

   /*** Get the wildcard string for use in matching userids. */
   mdtable  ("mai_Ainex", &inextab);
   if (chxtoken (wildcard, nullchix, 1, inextab) == -1)
      chxcpy (wildcard, CQ("*"));

   /*** Scan the ULIST. */
   allowed = 0;
   for (pos=0;   chxnextline (line, ulist, &pos);  ) {
      c1 = chxvalue (line, L(0));
      if (c1 == '#')  continue;

      if (c1 == ':') {
         type = tablematch (inextab, line);
         if (type < 1  ||  type > 4)  type = 0;
         continue;
      }

      chxcpy (group_file, line);              /* Preserve upper-case. */
      jixreduce    (line);
      chxsimplify  (line);
      match = 0;

      /*** If "<file" redirection, see if the groups file contains 
      /    a match. */
      if (c1 == '<') {
         chxclear  (name);
         chxcatsub (name, group_file, L(1), ALLCHARS);
         match = groupmatch (name, groups, userid, wildcard);
      }

      /*** Otherwise, this is a userid.  Was it a MATCH?  Allow for a
      /    wildcard at the end, e.g. "roth" matches "ro*" and everyone
      /    matches "*". */
      else  match = equal_id (line, userid, wildcard);

      /*** If this was a MATCH, set ALLOWED according to category! */
      if (match  &&  type > 0)   allowed = permission[type];
      
   }
   success = allowed;

done:
   chxfree (line);
   chxfree (name);
   chxfree (wildcard);
   chxfree (group_file);

   RETURN  (success);
}
