/*** GROUPMATCH.  Is USERID in this group file?
/
/    ok = groupmatch (file, groups, userid, wildcard);
/
/    Parameters:
/       int      ok;        (Success?)
/       Chix     file;      (name of group file)
/       Namelist groups;    (history of previous matches against GROUP files)
/       Chix     userid;    (looking for match against this userid)
/       Chix     wildcard;  (string used for wildcard matching)
/
/    Purpose:
/       Does the "groups file" FILE contain a match for USERID?
/
/    How it works:
/       Groupmatch() first verifies that FILE actually resides on
/       this host.  (FILE may contain an "@host" after the file
/       name proper.  If it's not on this host, return failure.)
/
/       Groupmatch then scans the namelist GROUPS to see if this
/       FILE has already been scanned for this userid.  If so, the
/       previous result is used.
/
/       Otherwise, groupmatch() proceeds to read FILE and scan it for a
/       match for USERID.  WILDCARD contains the string used for wildcard 
/       matching, i.e. if WILDCARD contains "*", then userid "ABCD" matches
/       FILE entry "AB*".  The result of this test is saved in GROUPS.
/
/       FILE may contain references to other files, e.g. "<subfile".
/       GROUPMATCH calls itself recursively to handle these cases.
/       If the file references itself, GROUPMATCH knows to avoid
/       infinite recursion.
/
/    Returns: 1 if a match for userid was found, else 0.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  allowed_on()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  conf/groupmat.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CR  7/17/90  9:45 New function. */
/*: CR  7/24/90 11:59 Avoid infinite recursion. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/25/91  1:11 Lower-case and trim trailing blanks from XSUG entries.*/
/*: CR  6/20/91 17:01 Use new equal_id() function. */
/*: CR  7/24/91 21:46 Remove unused variables. */
/*: CR 12/09/91 16:58 Handle FILE = "name@host"; expand header. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  3/05/92 11:28 Chixify. */
/*: DE  3/26/92 16:40 Patched chxindex w/ 0, */
/*: DE  5/26/92 13:33 Fix Chixification */
/*: JX  8/24/92 18:56 Fixify. */
/*: JV 10/09/92 15:09 Robustify. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"

extern union null_t null;
extern Chix         ss_inred;

FUNCTION  groupmatch (file, groups, userid, wildcard)
   Chix  file, userid, wildcard;
   struct namlist_t  *groups;
{
   struct namlist_t *searchlist, *f, *g, *nlnode();
   Chix   name, filename, host, thishost, format, message, tempname;
   int    match, lock, open, success;
   Flag   nullgroups;

   ENTRY ("groupmatch", "");

   host     = chxalloc (L(100), THIN, "host");
   name     = chxalloc (L(200), THIN, "name");
   format   = chxalloc (L(100), THIN, "format");
   message  = chxalloc (L(200), THIN, "message");
   filename = chxalloc (L(100), THIN, "filename");
   tempname = chxalloc (L(40),  THIN, "tempname");
   thishost = chxalloc (L(100), THIN, "thishost");

   if (groups == null.nl) {      /* Some ungrateful ingrate forgot to  */
      groups = nlnode (1);       /* pass us a namelist.  Make our own! */
      nullgroups = 1;
   } else nullgroups = 0;

   /*** If FILE is "filename@host", is "host" *this* host? */
   if (chxbreak (file, filename, host, CQ("@"))) {
      getmyhost (thishost);
      if (NOT chxeq (host, thishost))  FAIL;
   }

   /*** If this FILE is already in the GROUPS history, return
   /    the match value stored with it. */
   for (f=groups->next;   f!=null.nl;   f = f->next) {
      if (chxeq (f->str, filename))  {success = f->num; goto done;}
   }

   /*** If we can neither lock nor read the FILE, then complain, and
   /    add it to the GROUPS history with a match value of FALSE. */
   if (lock = unit_lock (XSUG, READ, L(0), L(0), L(0), L(0), filename)) 
         open = unit_view(XSUG);
   else  open = 0;

   if (NOT lock  ||  NOT open) {
      nlcat  (groups, file, 0);
/*    mdstr  (format, "mai_Fnogroup", null.md);
      chxformat  (message, format, L(0), L(0), file, null.chx);
      unit_write (XWER, message);
*/
      if (lock)  unit_unlk (XSUG);
      FAIL;
   }

   /*** Since FILE is not in GROUPS, we have to do the dirty work of
   /    actually looking through it.  SEARCHLIST is a list of subfiles
   /    that appear in FILE.  We don't actually search any of them until
   /    we're finished with FILE. */
   searchlist = nlnode (1);
   match      = 0;
   while (unit_read (XSUG, name, L(0))  &&  NOT match) {
      if (chxvalue (name, L(0)) == L('#'))  continue;
      jixreduce (name);
      chxtrim   (name);

      /*** If "<subfile" redirection, add "subfile" to the search list. */
      if (chxindex (name, L(0), ss_inred) == 0) {
         chxclear(tempname);
         chxcatsub(tempname, name, chxlen(ss_inred), L(200));
         nlcat (searchlist, tempname, 0);
         continue;
      }

      /*** Was this line a MATCH?  Allow for a wildcard at the end, e.g.
      /   "roth" matches "ro*" and everyone matches "*". */
      match = equal_id (name, userid, wildcard);
   }

   unit_close (XSUG);
   unit_unlk  (XSUG);

   /*** If we hit a match, we need to look no further. */
   if (match) {
      nlcat  (groups, filename, 1);
      nlfree (searchlist);
      SUCCEED;
   }

   /*** Temporarily add this file to the groups history list with a 
   /    result of 0.  This avoids infinite recursion if the file 
   /    somehow references itself.   Save the pointer to this entry
   /    in G, we'll just correct the result later. */
   nlcat (groups, filename, 0);
   for (g=groups->next;   g!=null.nl;   g = g->next)
      if (chxeq (g->str, filename))  break;

   /*** Otherwise, recursively look at the files added to this layer of the
   /    searchlist. */
   match = 0;
   for (f=searchlist->next;   f!=null.nl;   f=f->next) {
      if ( (match = groupmatch (f->str, groups, userid, wildcard)))  break;
   }
      
   /*** Success or no, add this file to the GROUPS history list. */
   nlfree (searchlist);
   g->num = match;
   success = match;

done:
   chxfree (host);
   chxfree (name);
   chxfree (format);
   chxfree (message);
   chxfree (filename);
   chxfree (tempname);
   chxfree (thishost);
   if (nullgroups) nlfree (groups);
   RETURN (success);
}
