
/*** LIMITUSERS.    Limit the number of simultaneous users.
/
/    ok = limitusers (getslot);
/
/    Parameters:
/       int    ok;          (success?)
/       int    getslot;     (1 => get a slot;  0 => free my slot)
/
/    Purpose:
/       Some versions of Caucus are sold with a license for a maximum
/       number of users.  If this is one of them, enforce that limit.
/   
/    How it works:
/       When GETSLOT is 1, LIMITUSERS attempts to assign a "slot" to
/       this user for the duration of the execution of the program.  If
/       a slot has already been assigned to this user, LIMITUSERS ensures
/       that no one can cheat by assigning one slot to multiple users.  If
/       the slot was successfully allocated (and no cheating has happened),
/       LIMITUSERS returns 1.  Otherwise, it returns 0.
/   
/       When GETSLOT is 0, LIMITUSERS frees the slot for use by another.
/       (Typically just before Caucus exits.)
/   
/       Each slot is associated with a file, usually called 'mxusr000 through
/       mxusrnnn.  The file is created with the time (returned by SYSTIME)
/       that the slot was allocated.
/
/    Returns: 1 on success
/             0 on no more slots, or detection of "cheating".
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  gen/limituse.c
/ */

/*** Copyright (C) 1987 Camber-Roth.   All rights reserved. */

/*: AA  7/29/88 17:41 Source code master starting point*/
/*: JV 11/26/90 16:03 Just thought I'd define GETSLOT... */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: JV 10/31/91 15:14 Declare sprintf(). */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  5/13/92 17:04 Fix nullchix. */
/*: JV  8/13/92 13:13 Declare other sprintf(). */
/*: CI 10/05/92 16:41 Chix Integration. */
/*: CR 12/11/92 15:33 Long chxint4 arg. */
/*: CR  1/26/93 15:49 Use PLAIN and strplain() to decrypt license info. */
/*: CK 10/14/93 15:14 Add hdr comments; use hash(uid) for start mxuser # */
/*: CK 10/15/93 15:24 Look for empty slot MAXTRY times; then look for reuse. */
/*: CR 10/17/93 17:54 Remove use of 'min()'. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#define   MAXTRY   10

extern struct license_t license;

FUNCTION  limitusers (getslot)
   int getslot;
{
   static short slot = -1;
   static int4  when;
   int4   crash, maxusers, value, hash, pos;
   int4   limitread(), limitslot(), systime();
   int    success, try, maxtry;
   Chix   uid;

   ENTRY ("limitusers", "");

   success    = 0;

   /*** If there's no limit on the number of users, always return success. */
   maxusers = PLAIN(license.maxusers);
   if (maxusers <= 0  ||  maxusers > 900)   SUCCEED;

   /*** Here if we're freeing our assigned slot, typically just before
   /    Caucus exits. */
   if (NOT getslot) {
      if (slot >= 0) {
         unit_lock (XSMU, WRITE, L(0), L(slot), L(0), L(0), nullchix);
         unit_kill (XSMU);
         unit_unlk (XSMU);
      }
      slot = -1;
      SUCCEED;
   }


   /*** If this is the first time LIMITUSERS is called, try to get a
   /    SLOT reserved for this user.  If we succeed, remember
   /    the slot number and WHEN we got the slot. */
   if (slot < 0) {

      /*** Hash this user's userid into a "random" number. */
      uid = chxalloc (L(40), THIN, "limituse uid");
      sysuserid (uid);
      for (hash=0, pos=0;   value = chxvalue (uid, pos);   pos++)
         hash += value;
      chxfree (uid);

      /*** Go through the first MAXTRY possible slots, staring at "random"
      /    spot computed from a hash of this userid. */
      maxtry = (maxusers < MAXTRY ? maxusers : MAXTRY);
      for (try=0;   try<maxtry;   ++try) {
         slot = (try + hash) % maxusers;
         if ( (when = limitslot (slot)) > 0)  SUCCEED;
      }

      /*** No empty slots.  See if any of them are more than 12 hours
      /    old.  If so, reuse it--it's probably left over from a crash. */
      crash = systime() - L(12) * L(3600);
      for (try=0;   try<maxusers;   ++try) {
         slot = (try + hash) % maxusers;
         if (limitread (slot) > crash)  continue;
         if (unit_lock (XSMU, WRITE, L(0), L(slot), L(0), L(0), nullchix)) {
            unit_kill (XSMU);
            unit_unlk (XSMU);
         }

         if ( (when = limitslot (slot)) > 0)   SUCCEED;
      }
      slot = -1;

      FAIL;
   }


   /*** On subsequent calls, we're just checking to make sure nobody
   /    has diddled with our SLOT.  Read the XSMU file to make sure
   /    the time it was allocated hasn't changed. */
   DONE(limitread (slot) == when);

 done:

   RETURN ( success );
}


/*** LIMITREAD.   Return the time stored in XSMU file N.
/    If the file cannot be read, return -1.  */

FUNCTION  int4 limitread (n)
   int    n;
{
   int4   startpos;
   int4   then;
   Chix   str;

   ENTRY ("limitread", "");
   str = chxalloc (L(40), THIN, "limitread str");

   then = -1;
   if (unit_lock (XSMU, WRITE, L(0), L(n), L(0), L(0), nullchix)) {
      if (unit_view (XSMU)) {
         startpos = 0;
         if (unit_read (XSMU, str, L(0)))  then = chxint4 (str, &startpos);
         unit_close (XSMU);
      }
      unit_unlk (XSMU);
   }

   chxfree ( str );

   RETURN  (then);
}


/*** LIMITSLOT.  Try to use slot N for this user. */

FUNCTION  int4 limitslot (n)
   int    n;
{
   int4   now, systime();
   Chix   str;
   int    success;

   ENTRY ("limitslot", "");

   str     = chxalloc (L(40), THIN, "limitslot str");
   success = 0;

   if (NOT unit_lock (XSMU, WRITE, L(0), L(n), L(0), L(0), nullchix))  FAIL;

   now = L(0);
   if (NOT unit_check (XSMU)  &&  unit_make (XSMU)) {
      now = systime();
      chxformat (str, CQ("%12d\n"), now, L(0), nullchix, nullchix);
      unit_write (XSMU, str);
      unit_close (XSMU);
   }
   unit_unlk (XSMU);
 
   DONE(now);

 done:

   chxfree ( str );

   RETURN  ( success );
}
