
/*** SENDWHO.   Send this message to WHO?
/
/    ok = sendwho (prompt, address, export);
/
/    Parameters:
/       int       ok;       (success, return-receipt requested?)
/       char     *prompt;   (prompt string to use to ask for a recipient)
/       Address   address;  (resulting address)
/       int      *export;   (value of SET EXPORT for recipient)
/
/    Purpose:
/       During the process of sending a Caucus message, there is a dialogue
/       between Caucus and the user about the identity of the recipient.
/       Sendwho() is responsible for handling that dialogue.  It asks up
/       to three questions:
/         1. What is (a part of) the recipient's name?
/         2. If there are multiple matches, which is the real recipient?
/         3. Final confirmation to send message to that person.
/
/    How it works:
/       If ADDRESS is empty, sendwho() asks for the recipient's name
/       (question 1 above) with PROMPT.
/
/       Sendwho() then calls choose_name() to reduce ADDRESS to a
/       single person, interacting with the user if necessary.
/
/       Lastly, sendwho() prompts for final confirmation, using the
/       person's full name (if it is available).  The complete address
/       is placed back in ADDRESS.
/
/       If the selected user has "SET EXPORT EXPORT", EXPORT is set to 1.
/       Otherwise, it is set to 0.
/
/    Returns: 2 if person selected & confirmed, and return-receipt requested.
/             1 if person selected & confirmed
/             0 if no person was selected
/            -1 if userid selected by "(userid)" form, but no such userid.
/               (offers caller possibility to send message to self)
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  mess_send()
/
/    Operating system dependencies:
/
/    Known bugs:
/       The "There is no userid" message should come from the dictionary.
/
/    Home:  mess/sendwho.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:49 Source code master starting point */
/*: JV  9/05/89 17:46 toomany() --> nlunique() for user selection. */
/*: CW  5/18/89 15:07 Add XW unit codes. */
/*: CR  8/31/89 15:24 Add code for return-receipt. */
/*: JV  1/01/90 23:00 Add code for Caucus Mailer Interface. */
/*: CR  1/26/90 16:51 Add distribution lists via "<file" at ENTER RECIP. */
/*: CR  1/28/90  2:55 "=name" always returns success. */
/*: JV  2/06/90 12:25 Changed printing of mes_Fsndun message. */
/*: CR  2/03/90 17:29 Remove "how". */
/*: CR  5/08/90 17:03 Put "how" back in!  Fix %$*^! comment that fooled VM.*/
/*: CJ 11/21/90 18:07 Allow CMI only if function_mask bit set. */
/*: CR  1/31/91 18:04 Use person_format() to display person's name. */
/*: CR  3/29/91 17:30 Use AND/OR code for matchnames() call. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  7/16/91 12:02 Use new address argument, add new header. */
/*: CR  7/16/91 17:08 Add HOST argument to person_format() calls. */
/*: CR  7/30/91 18:14 Add 3rd arg to loadreg(). */
/*: CR 10/10/91 22:09 Complain if RECIP doesn't match anything. */
/*: CR 11/07/91 14:18 Add confnum arg to matchnames() call. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 int4.*/
/*: CR 12/20/91 13:09 Add ERROR argument to choose_name. */
/*: DE  3/30/92 15:09 Chixified */
/*: DE  4/10/92 14:19 Converted sysfree -> chxfree */
/*: JX  5/27/92 13:22 Declare make_address(), fix NULLADDRESS arg. */
/*: CX  6/01/92 12:11 make_address() -> into_address(); add make_userreg(). */
/*: JX  6/18/92 16:32 Only free who->briefs,print once: in free_userreg(). */
/*: JX  6/23/92 16:32 Only free result once. */
/*: CR 10/08/92 14:37 Fix replies to CMI to use address->name, not recip. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CR  1/26/93 15:49 Use PLAIN and strplain() to decrypt license info. */
/*: CR  2/05/93 15:17 Always free 'result' before getting a new one. */
/*: CR  2/25/93 23:09 Add 'export' argument. */
/*: CW  7/06/93 12:59 Add new -1 return code for bad "(userid)" forms. */
/*: CK  9/20/93 14:00 Add run_prompt_macro() call. */

#include <stdio.h>
#include "caucus.h"

extern struct license_t license;
extern Userreg          thisuser;
extern int              debug;
extern Chix             yesnotable;
extern Chix             ss_dotnl, ss_cmiesc, ss_inred, newuseropts;
extern Chix             confname;
extern union null_t     null;

FUNCTION  sendwho (prompt, address, export)
   char   *prompt;
   Address address;
   int    *export;
{
   static  Chix  oksend;
   static  Flag  dist_list = 0;

   Userreg who, make_userreg();
   int     how, error, success, nosuchid;
   Address pattern, make_address();
   Chix    recip, answer, yesses, yes, result, thishost, host;
   Chix    format, newstr, tempchx;
   Chix    person_format();

   ENTRY ("sendwho", "");

   recip     = chxalloc (L(62),  THIN, "sendwho recip");
   answer    = chxalloc (L(12),  THIN, "sendwho answer");
   yesses    = chxalloc (L(40),  THIN, "sendwho yesses");
   yes       = chxalloc (L(20),  THIN, "sendwho yes");
   result    = chxalloc (L(100), THIN, "sendwho result");
   thishost  = chxalloc (L(100), THIN, "sendwho thishost");
   host      = chxalloc (L(100), THIN, "sendwho host");
   format    = chxalloc (L(80),  THIN, "sendwho format");
   newstr    = chxalloc (L(80),  THIN, "sendwho newstr");
   pattern   = make_address();
   who       = make_userreg();
  *export    = 0;
   success   = 0;

   /*** Loop forever.  Return only if we got a name the user ok'd, or
   /    the user doesn't want to send this message to any more people. */
   while (1) {

      /*** Prompt the user until we've got a good address... */
      while (NULLADDRESS (address)) {
   
         /*** If we're reading names from a distribution list, get the next
         /    one.  When no more are left, close the file. */
         if (dist_list) {
            if (NOT (dist_list = unit_read (XIRE, recip, L(0))) ) {
               unit_close (XIRE);
               unit_unlk  (XIRE);
            }
         }
   
         /*** If we're NOT reading from a distribution list, prompt the
         /    user for a name.  If the name is itself a distribution list,
         /    open it and read the first name. */
         if (NOT dist_list) {
            if (mdprompter (prompt, null.chx, 0, "mes_Hsendwho",
                            60, recip, null.chx) < -1)
               FAIL;
         
            /*** Is this a distribution list, e.g. did the user type
            /    "<file" at ENTER RECIPIENT? */
            if (chxindex (recip, L(0), ss_inred) == 0) {
               if (unit_lock (XIRE, READ, L(0), L(0), L(0), L(0),
                              tempchx =  CHXSUB(recip,chxlen(ss_inred)))) {
                  dist_list = unit_view (XIRE);
                  if (NOT dist_list)  unit_unlk (XIRE);
               }
               chxfree ( tempchx );
               if (NOT dist_list) {
                  mdwrite (XWER, "mes_Enodist", null.md);
                  unit_write (XWER, tempchx = CHXSUB(recip,chxlen(ss_inred)));
                  chxfree ( tempchx );
                  unit_write (XWER, ss_dotnl);
               }
               continue;
            }
         }
   
         into_address (recip, pattern);
         if (NOT choose_name (pattern, address, &error)) {
            if (error == NOPERSON) {
               mdwrite (XWER, "mes_Enopart", null.md);
               unit_write (XWER, recip);
               unit_write (XWER, ss_dotnl);
            }
            continue;
         }
      }
      nosuchid = 0;
   

      /*** If CMI mail, just acknowledge address. */
/*    if (chxindex (recip, L(0), ss_cmiesc) == 0) [  */
      if (chxindex (address->name, L(0), ss_cmiesc) == 0) {
         if (NOT (PLAIN(license.function_mask) & FM_CMI))  FAIL;
         mdstr   (format, "mes_Fsndun", null.md);
         chxclear ( newstr );
         chxformat (newstr, format, L(0), L(0), tempchx = CHXSUB(address->name,1L),
                         null.chx);
         chxfree ( tempchx );
         chxconc (newstr, newstr, CQ("\n"));
         unit_write (XWTX, newstr);
         if (debug) unit_write (XWTX, CQ("Leaving sendwho.\n"));
         SUCCEED;
      } 

      /*** If user has selected a local person, get their full name and
      /    personal info in WHO.  Otherwise, fill WHO in from ADDRESS. */
      chxclear ( who->phone );
      getmyhost (thishost);
      if (NOT EMPTYCHX(address->id)  &&
           (EMPTYCHX(address->host)  ||  chxeq (address->host, thishost))) {
         if (NOT loadreg (address->id, who, NULLSITE)) {
            unit_write (XWER, CQ("\nThere is no userid '("));
            unit_write (XWER, address->id);
            unit_write (XWER, CQ(")'.\n"));
            sysuserid  (who->id);
            loadreg    (who->id, who, NULLSITE);
            nosuchid = 1;
         }
         decode_setopts  (who, who->setopts, nullchix, newuseropts);
        *export = who->export_mail;
         chxclear ( host );
      }
      else {
         chxcpy (who->name, address->name);
         chxcpy (who->id,   address->id);
         chxcpy (host,     address->host);
      }

      /*** OK to send to .... ?  */
      mdtable  ("mes_Aoksend", &oksend);      /* Answers to the question. */
      chxtoken (yesses, null.chx, 2, oksend);
      chxbreak (yesses, yes, format, CQ("/"));
      mdstr    (format, "mes_Foksend", null.md);  /* Format of question. */
      chxfree  (result);
      result = person_format (format, who->name, who->id, host, 
                                      who->phone, who->briefs);
      run_prompt_macro ("mes_Foksend", thisuser->dict, confname);
      how = prompter (result, oksend, 2, "mes_Hoksend", 10, answer, yes);
      if (how < 0)                FAIL;
      if (how > 0  &&  nosuchid)  DONE(-1);
      if (how > 0)                DONE(how);

      into_address (null.chx, address);
   }

done:
   free_userreg (who);
   free_address (pattern);
   chxfree (recip);
   chxfree (answer);
   chxfree (yesses);
   chxfree (yes);
   chxfree (result);
   chxfree (thishost);
   chxfree (host);
   chxfree (format);
   chxfree (newstr);

   RETURN ( success );

}
