
/*** SYSMDSTR.  Fetch a dictionary string into a chix.
/
/    sysmdstr (name, dicnum, a, status);
/
/    Parameters:
/       char *name;    (name of dictionary string)
/       int   dicnum;  (dictionary number)
/       Chix  a;       (place to put resulting chix)
/       int  *status;  (place to put status bits for string)
/
/    Purpose:
/       Fetch dictionary string NAME from dictionary DICNUM, and 
/       put it in chix A.  Put the status bits for this string in
/       STATUS.
/
/    How it works:
/       Make sure dictionary DICNUM is open.  Hash NAME and get
/       the header record from the dictionary hash table.  Seek
/       to the beginning of the text of the string.  Read the text
/       into a temporary buffer, then convert it to chix A.
/
/       If dictionary DICNUM cannot be opened, use dictionary 0.
/
/       If DICNUM < 0, close any open dictinary(ies), and just return.
/
/    Returns: 1 if successful, 0 on error.
/
/    Error Conditions:
/       Cannot open dictionary DICNUM *or* 0.
/       String NAME not in dictionary.
/       Cannot allocate enough memory for string.
/       Chxofraw() fails (A not chix, not enough memory).
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  mdstr()
/
/    Operating system dependencies: "file descriptor" open, read & seek calls.
/
/    Known bugs:
/       No code for server (VM/CMS) version.
/
/    Home:  sys/sysmdstr.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */
/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: CX  5/09/92 19:28 New function. */
/*: NT  4/08/97 16:00 Revamp for Caucus 3.2. */

#include <stdio.h>
#include <sys/types.h>
#include "caucus.h"
#include "diction.h"
#include "systype.h"

#if UNIX
#define  OPEN(file)   open (file, 0)
#define  SEEK(pos)    lseek(mdfd, (off_t) pos, 0)
#define  CLOSE        close(mdfd)
#define  MREAD(a,b,c) read(a,b,c)
#endif

#if NUT40
#include <fcntl.h>
#define  OPEN(file)   open (file, O_RDONLY | O_BINARY)
#define  SEEK(pos)    lseek(mdfd, pos, 0)
#define  CLOSE        close(mdfd)
#define  MREAD(a,b,c) read(a,b,c)
#endif

#if WIN32
#include <fcntl.h>
#define  OPEN(file)   _open (file, O_RDONLY | O_BINARY)
#define  SEEK(pos)    _lseek(mdfd, pos, 0)
#define  CLOSE        _close(mdfd)
#define  MREAD(a,b,c) _read(a,b,c)
#endif

extern int  debug;

FUNCTION  sysmdstr (name, dicnum, a, status)
   char  *name;
   int    dicnum;
   Chix   a;
   int   *status;
{
   char  *bugtell();

#if UNIX | NUT40 | WIN32

#define  DICTSHIFT    0

#define  BUFSIZE    512
   static short thisdict = -1;
   static int   mdfd = -1;
   static char  buf[BUFSIZE];
   char   hashname[DICNAMELEN+20];
   int4   hash, hashbyte;
   int    code;
   struct mdrec_t mdrec;
   char  *s, *block, *sysmem();

   ENTRY ("sysmdstr", "");

   /*** Close dictionary if DICNUM < 0. */
   *status = 0;
   if (dicnum < 0  &&  mdfd >= 0)       { CLOSE;   RETURN(1); }
   if (dicnum < 0  ||  NULLSTR(name))              RETURN(1);

   /*** If dictionary DICNUM is not open, open it! */
   if (thisdict != dicnum + DICTSHIFT) {
      if (mdfd >= 0)  CLOSE;
      mdfd = opendict (dicnum + DICTSHIFT);
      if (mdfd <  0)  mdfd = opendict (0 + DICTSHIFT);
      if (mdfd <  0)  RETURN (0);
      thisdict = dicnum;
   }

   /*** Hash NAME into an address in the Master Dictionary. */
   strcpy (hashname, name);
   hashname[DICNAMELEN] = '\0';
   for (hash=0L, s=hashname;   *s;   ) {
      hash += (*s++ & 0xFF) * L(256);
      if (NOT *s)  break;
      hash += (*s++ & 0xFF) *   L(5);
   }
   if (debug & DEB_DICT)
      bugtell ("MDPS", thisdict, hash % HASHSIZE, hashname);

   /*** Seek to that address, and read the MDREC.  If a collision occurs,
   /    try the next record. */
   do {
      hashbyte = (hash++ % HASHSIZE) * (int4) sizeof(struct mdrec_t);
      SEEK (hashbyte);
      MREAD (mdfd, (char *) &mdrec, sizeof(mdrec));
      if (NULLSTR (mdrec.name))  RETURN (0);
   } UNTIL (streq (hashname, mdrec.name));

   SEEK (mdrec.start);

   /*** Actually read the string into a buffer, and then put the
   /    buffer into chix A. */
   block = (mdrec.length < BUFSIZE ? buf : sysmem (L(mdrec.length)));
   if (block == NULL)  RETURN (0);
   MREAD (mdfd, block, mdrec.length);
   code = chxofraw (a, block, mdrec.width, L(mdrec.length));
   if (block != buf) sysfree (block);
   *status = mdrec.status;

   RETURN (code);
}


FUNCTION opendict (n)
   int   n;
{
   char  mdfile[200];
   char *bugtell();

   unit_name (mdfile, XSMD, L(0), L(n), L(0), L(0), nullchix);
   if (debug & DEB_DICT) bugtell ("OPENDICT: dict=\n", 0, L(0), mdfile);
   return (OPEN (mdfile));
}
#endif
