
/*** INS2.   Program to perform phase 2 of a Caucus install on Windows/NT.
/
/    Invoked as:
/       http://localhost:8022/cgi-bin/ins2.exe?page[+child+tempdir]
/
/    Arguments:
/       page     "page number" for ins2 to produce
/       child     process id of web server
/       tempdir   full path to temporary directory where install files live
/
/    Purpose:
/       First page/program of browser-based Caucus install
/
/    How it works:
/       On first use, WEB and TEMPDIR must be specified.  Afterwards,
/       they are passed from page to page with other variables as
/       HIDDEN form elements.
/
/       INS2 acts as one huge state table.  Each page has a number
/       (e.g., 100, 200, 210).  The "processing" (form target) for
/       each page is that page number + 1.  The processing section
/       in INS2 then redirects to the appropriate page number.
/       (For example, page 100 asks for the activation key.  "Page" 101
/       decides what to do with the results; it send the browser on to
/       page 110 if the activation key was invalid.)
/
/       Page numbers:
/         100 Initial page, asks for activation key.
/         101 check key; look for previous caucus installs, => 2xx.
/         110 Err: Invalid activation key
/         120 Err: Expired activation key
/         130 Err: no NTFS drives
/
/         210 Offer to create new userid or use existing one
/         220 Offer to load over top of existing caucus installation
/         230 Offer to load over top of 1 of several existing installations
/
/         240 Create new userid
/         250 Use existing userid
/
/         270 not enough space to create userid
/         280 Actually do userid creation (from 240)
/         281 Wait for creation to complete
/
/         290 branch: check for enough free disk on selected drive
/         295 complain about insufficient space for existing id
/
/         300 Continue on -- ask for hostname.
/         310 insufficient disk -- allow cleanup, or go back to 210
/
/         900 Browsing sub-window -- find new home directory for caucus
/
/    Exit status:
/
/    Known bugs:      none
/
/    Home:  xins/ins2.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/10/98 14:28 1st version. */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <windows.h>
#include <iostream.h>
#include "ins.h"

#define  MINDISK  20000

FUNCTION main (int argc, char *argv[])
{
   char   value[200], pass[80], ship[80], code[80], expire[80], users[80];
   char   result[600], dostemp[200];
   struct vars_t vars;
   int    slot, rc, page, childpid, ct;
   char   tempdir[200] = {0};
   char   child  [ 80] = {0};
   char   bpath  [400] = {0};
   char   temp  [2000], cmd[400];
   char   drives[80];
   int    freesp[80], len, is_ie, loop, freedisk, found;
   char  *p, drivelet;
   char  *valueof(), *dblslash();
   void   logger();
   HANDLE hChild;

   /*** Initialize the variable table, read all forms data into it. */
   vars.max = V_MAX;
   parse_form (&vars);

   /*** Find out which browser we've got. */
   env_var (value, "HTTP_USER_AGENT");
   is_ie = strindex (value, "MSIE") >= 0;

   /*** Parse the argument string.  There are three cases:
   /    (a) page 100, first call; get child (pid of web server) and
   /        tempdir, put them in vars so abort knows what to do.
   /    (b) page 900 (pop-up browse-file-system), pluck out the path
   /    (c) all other cases, error message if we lost child or tempdir. */
   env_var (value, "QUERY_STRING");
   sscanf  (value, "%d", &page);
   if (page == 100  &&  value[3] == '+') {
      strcpy (child,   value+4);    child[7] = '\0';
      addvar (&vars, "child",   child); 
      addvar (&vars, "tempdir", value+12);
   }
   if (page == 900  &&  value[3] == '+')   qdecode (bpath, value+4);
   else {
      if (NULLSTR(valueof("child",   &vars))  || 
          NULLSTR(valueof("tempdir", &vars))) {
         htop ("Caucus Installation Script Error", "");
         printf ("<P>Error 201: There was an internal error in the \n");
         printf ("installation scripts that 'lost' the values of\n");
         printf ("'child' and 'tempdir'.  The value of Page = %d.\n"
                 " Please report to Caucus Systems Inc. at\n", page);
         hsupport();
         printf (".\n</BODY></HTML>\n");
         exit   (0);
      }
   }
      

   /*-----Rest of the code handles each individual page...---------------*/
   
   /*** Loop over state transitions.  There are never more than 4 such
   /    transitions in a single call. */
   for (loop=0;   loop < 4;   ++loop) {

      /*** Major step 1 is about getting the activation key, and checking
      /    that enough things (key, disks) are OK to continue. */
      if (page == 100)   ci_activ (page, &vars);
   
      if (page == 101) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
   
         /*** Test activation key. */
         rc = goodkey (valueof("actkey", &vars), pass, ship, code, 
                                                 expire, users);
         if (NULLSTR(valueof("actkey", &vars)))  rc = -2;
         if      (rc == -1)  page = 120;
         else if (rc == -2)  page = 110;
   
         /*** Verify there's at least one NTFS drive. */
         else if (NOT get_ntfs (drives, freesp)  ||  
                  (len = strlen(drives)) == 0)   page = 130;
   
         /*** Try to find a previous installation(s) of Caucus. */
         else if ( (found = find_caucus (result, drives)) ) {
            if (found > 1)                  page = 230;
            else                            page = 220;
         }
         else                               page = 210;
      }
   
      if (page == 110)  ci_invalid (page, &vars);
      if (page == 111) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         page = 100;
      }
   
      if (page == 120)  ci_expire  (page, &vars);
      if (page == 121) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         page = 100;
      }
   
      if (page == 130)  ci_nontfs  (page, &vars);
   
   
      /*** Major step 2 is about selecting the desired userid and
      /    home directory, where we'll install caucus. */

      /*** Page 210, create new id or re-use existing one. */
      if (page == 210)   ci_whatid (page, &vars);

      if (page == 211) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);

         if      (streq (valueof ("submit", &vars), " A "))  {
            clearvalue ("home", &vars);   
            clearvalue ("uid",  &vars);
            page = 240;
         }
         else if (streq (valueof ("submit", &vars), " B "))  page = 250;
/*       else if (streq (valueof ("submit", &vars), " C "))  page = 260; */
         else {
            sprintf (temp, "valueof(submit)='%s'", valueof("submit", &vars));
            ci_bad  (page, &vars, temp);
         }
      }
   
      /*** Previous caucus installation(s) found, offer to load over
      /    one of them. */
      if (page == 220  ||
          page == 230)      ci_found  (page, &vars, result);
   
      if (page == 221  ||  page == 231) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         if (streq (valueof("submit", &vars), "No"))     page = 210;
         else                                            page = 290;
      }
   
      if (page == 240)   ci_create (page, &vars); 
      if (page == 241) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);

         get_ntfs (drives, freesp);
         
         ct = which_disk (drives, &vars);
         freedisk = (ct >= 0 ? freesp[ct] :  0 );
         drivelet = (ct >= 0 ? drives[ct] : '#');

         if (freedisk < MINDISK)  page = 270;
         else                     page = 280;
      }

      /*** Offer list of existing userids to pick from. */
      if (page == 250)  ci_useold (page, &vars);
      if (page == 251) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         if (streq (valueof("submit", &vars), "Back" ))  page = 101;
         else {
            nt_uinfo (valueof("uid", &vars), "home directory", tempdir);
            setvalue ("home", &vars, tempdir);
            page = 290;
         }
      }

      /*** Actually create the new userid. */
      if (page == 280) {
         sprintf  (cmd, "net user %s /add /expires:never /fullname:\"Caucus\" "
                    "/homedir:\"%s\" \"%s\"", valueof("uid",    &vars),
                    valueof("home", &vars),   valueof("passwd", &vars));
         shell    (temp, 2000, cmd);
         sysmkdir (valueof("home", &vars), 0700);
         Sleep (5000);
         page = 281;
      }

      /*** Waiting for the userid to finish being created... */
      if (page == 281)  {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         if (was_created (valueof("uid", &vars)))        page = 300;
         else                                    ci_crewait (page, &vars);
      }

      if (page == 270)  ci_insuf (page, &vars, freedisk, drivelet, MINDISK);
      if (page == 271)  {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         page = 240;
      }

      /*** Enough space to install in existing userid? */
      if (page == 290) {
         /*** Find out the driveletter for where we're installing Caucus,
         /    and how much free space is on that drive. */
         get_ntfs (drives, freesp);
         ct = which_disk (drives, &vars);
         freedisk = (ct >= 0 ? freesp[ct] :  0 );
         drivelet = (ct >= 0 ? drives[ct] : '#');

         if (freedisk < MINDISK)  page = 295;
         else                     page = 300;
      }
      if (page == 295)   ci_nospace (page, &vars, freedisk, drivelet, MINDISK);
      if (page == 296) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
         page = 101;
      }
     

      if (page == 300)   ci_host    (page, &vars);
      if (page == 301) {
         if (streq (valueof("submit", &vars), "Abort"))  ci_abort (&vars);
      }

   
      if (page == 900)   ci_browse  (page, &vars, bpath, is_ie);
   }

   ci_bad (page, &vars, "(at bottom)");
   exit (0);
}


FUNCTION  char *valueof (char *name, struct vars_t *vars)
{
   static char empty[2] = {0};
   int    slot;

   if (vars == NULL)  return ("No 2nd arg!");

   for (slot=0;   slot<vars->used;   ++slot) {
      if (streq (name, vars->name[slot]))   return (vars->value[slot]);
   }

   return (empty);
}

FUNCTION  clearvalue (char *name, struct vars_t *vars)
{
   int    slot;

   if (vars == NULL)  return (0);

   for (slot=0;   slot<vars->used;   ++slot) {
      if (streq (name, vars->name[slot]))  {
         vars->value[slot][0] = '\0';
         return (1);
      }
   }

   return (0);
}

FUNCTION  setvalue (char *name, struct vars_t *vars, char *value)
{
   int    slot;

   if (vars == NULL)  return (0);

   for (slot=0;   slot<vars->used;   ++slot) {
      if (streq (name, vars->name[slot]))  {
         strcpy (vars->value[slot], value);
         return (1);
      }
   }
   strcpy (vars->value[slot], value);
   ++vars->used;

   return (1);
}


FUNCTION  void logger (char *str)
{
   FILE  *fp;

   if ( (fp = fopen ("c:/xins/log", "a")) == NULL)  return;

   fprintf (fp, "%s", str);
   fclose  (fp);
   return;
}

