/*** LICEPASS4.   Compute licensing password.  Caucus 4.0
/
/    Invoked as:
/       licepass4   ship  code  options
/
/    Arguments:
/       ship       first part of serial code, e.g. "TY01".
/       code       numeric part of serial code, e.g. "023".
/       options    licensing options.
/
/    Purpose:
/       LICEPASS4 computes the licensing password needed by program
/       LICE4.  It contains no user interface code, and is meant to 
/       be run from a shell script that asks all of the questions 
/       about how Caucus should be licensed for a particular customer.
/
/    How it works:
/       Any or all of the following "options" may be specified.  The
/       "-s" option may be specified more than once to allow multiple
/       sysids.
/
/          -e days         license expires in this many days from now.
/          -s id           license runs on this system id.
/          -f mask         specify function mask value as integer
/          -u users        specify maximum number of simultaneous users
/          -c confmax      max number of conferences
/          -i itemmax      max number of items
/          -r respmax      max number of responses
/          -t totalusers   total # of registered users, max
/
/       Licepass writes the computed password on stdout.
/
/    Exit status:
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xlice/licepass4.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  1/24/93 12:03 1st carvmark */
/*: CK 10/13/93 17:33 Add confmax, itemmax, respmax. */
/*: JZ 10/18/93 11:00 Declare strcpy() */
/*: CR  2/03/95 16:02 Initialize confmax, itemmax, respmax! */
/*: CR 10/30/95 11:01 Add totalusers, -t. */
/*: CR  3/05/98 14:23 New version for Caucus 4.0 */

#include <stdio.h>
#include "lice4.h"
#include "int4.h"

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   char   result[80], *strcpy();
   int4   expires_in, mask, users, confmax, itemmax, respmax, totalusers;
   int    i;

   if (argc < 4) {
      fprintf (stderr,
         "Usage: licepass4 ship code [-e days] [-s id] [-f mask] [-u users]\n");
      exit (1);
   }

   /*** Start with "abcdefg".  Permute by SHIP and CODE. */
   strcpy      (result, "abcdefg");
   lstrpermute (result, argv[1]);
   lstrpermute (result, argv[2]);

   /*** Permute by expiration, mask, and users. */
   expires_in = mask = users = confmax = itemmax = respmax = totalusers = 0;
   for (i=3;   i<argc;   ++i) {
      if (streq (argv[i], "-e"))  sscanf (argv[++i], "%d", &expires_in);
      if (streq (argv[i], "-f"))  sscanf (argv[++i], "%d", &mask);
      if (streq (argv[i], "-u"))  sscanf (argv[++i], "%d", &users);
      if (streq (argv[i], "-c"))  sscanf (argv[++i], "%d", &confmax);
      if (streq (argv[i], "-i"))  sscanf (argv[++i], "%d", &itemmax);
      if (streq (argv[i], "-r"))  sscanf (argv[++i], "%d", &respmax);
      if (streq (argv[i], "-t"))  sscanf (argv[++i], "%d", &totalusers);
   }
   lnumpermute (result, expires_in / 30);
   lnumpermute (result, expires_in % 30);
   lnumpermute (result, mask);
   lnumpermute (result, users);
   lnumpermute (result, confmax);
   lnumpermute (result, itemmax);
   lnumpermute (result, respmax);
   lnumpermute (result, totalusers);

   /*** Permute by each sysid. */
   for (i=3;   i<argc;   ++i)
      if (streq (argv[i], "-s"))
         lstrpermute (result, argv[++i]);

   printf ("%s\n", result);
}
