
/*** LICE41.    New Caucus licensing program.  (Caucus 4.1)
/
/    Invoked as:
/       lice41   caucus  password  customer os options
/
/    Arguments:
/       caucus     name of caucus_x file.
/       password   licensing password as rendered by licepass.
/       customer   five digit customer number
/       os         two letter operating system code
/       options    licensing options.
/
/    Purpose:
/       Lice41 is the one-stop licenser for all Caucus licenses.  It
/       contains no user interface code, and is meant to be run from
/       a shell script that asks all of the questions about how Caucus
/       should be licensed for a particular customer.
/
/       Because Lice41 *requires* a password, it can double as a
/       re-licenser that can be shipped with Caucus to a customer site.
/
/    How it works:
/       Any or all of the following "options" may be specified.  The
/       "-s" option may be specified more than once to allow multiple
/       sysids.
/
/          -w weeks        license expires this many weeks from now.
/          -s id           license runs on this system id.
/          -u users        specify maximum number of simultaneous users
/          -a address      specify address of license block
/          -f mask         specify function mask value as integer
/          -c confmax      max number of conferences
/          -i itemmax      max number of items
/          -r respmax      max number of responses
/          -t totalusers   max total number of registered users
/
/          -b              reverse byte order for integers
/          -8              use 8 byte int4 integers
/
/       Unix versions of Lice41 can scan caucus_x for the license block
/       locator and thus do not need -a.  If it is supplied, the address
/       argument must be decimal.
/
/       The meaning of the bits in "mask" are as follows:
/          1     On => Allow CMI
/          2     On => Allow CaucusLink
/          4     On => Display Caucus copyright notice
/
/       The values of "weeks", "id", "users", "confmax", "itemmax", "respmax",
/       "totalusers", and "mask" are stored in an encrypted form in the 
/       license block in caucus_x.  "Address" is never stored.
/
/       When Lice41 is finished, it writes the licensing information to stdout.
/       The first line contains the max users, expiration date, and serial code.
/       Subsequent lines contain the system id(s).
/
/    Exit status:
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  xlice/lice41.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved.  */

/*: CR  1/14/93 12:03 New program. */
/*: CR  1/26/93 16:15 Add encryption stuff (PLAIN, strplain). */
/*: CP  4/01/93 11:43 Add Vax/VMS support. */
/*: CV  6/15/93 17:31 Map " into + for Vax. */
/*: CK 10/13/93 17:33 Add confmax, itemmax, respmax. */
/*: CR 10/17/93 17:56 Rename search() -> searchfile(). */
/*: JZ 10/18/93 11:00 lseek arg3 is int, not int4; declare strcpy(). */
/*: CR 11/27/94 17:16 Add "-b" byte-order reversing option. */
/*: CR  2/03/95 16:02 Initialize confmax, itemmax, respmax! */
/*: CR 10/30/95 10:51 Add totalusers, -t. */
/*: CR  3/05/98 14:23 New version for Caucus 4.0 */
/*: CR  3/27/99 14:15 New version for Caucus 4.1 */

#include <stdio.h>
#include "lice41.h"
#include "license.h"
#include "systype.h"

#define MAXSYSID 16

#if UNIX
#define  OPEN(x)       open (x, 2)
#define  CLOSE(x)      close(x)
#define  LSEEK(x,y,z)  lseek (x, y, z)
#endif

#if WIN32
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#define  OPEN(x)       _open (x, O_BINARY | O_RDWR)
#define  CLOSE(x)      _close(x)
#define  LSEEK(x,y,z)  _lseek(x, y, z)
#endif

#define  REV(x)    (reverse ? rev4(x) : x)

static struct license_t license =
   {0x11, 0x22, 0x33, 0x44, 0xAA, 0xBB, 0xCC, 0xDD};

static struct license8_t {
   char   locator[8];
   int4   expires[2];
   int4   maxusers[2];
   int4   function_mask[2];
   int4   confmax[2];
   int4   itemmax[2];
   int4   respmax[2];
   int4   totalusers[2];
   int4   reserve2[2];
   char   serial[16];
   char   sysid[16][16];
} licens8 = {0x11, 0x22, 0x33, 0x44, 0xAA, 0xBB, 0xCC, 0xDD};

FUNCTION  main (argc, argv)
   int   argc;
   char *argv[];
{
   FILE *pf;
   char  caucus_x[200], result[40], result2[40], pfile[200];
   int4  address;
   int4  mask, expires_in, users, confmax, itemmax, respmax, totalusers;
   int4  searchfile(), systime();
   int   fd, i, j, reverse, long8;
   char  str[40];
   char *strplain(), *strcpy();
   int4  rev4();

#if UNIX | WIN32
   if (argc < 5) {
      printf ("Usage: lice41 caucus_x customer os options\n");
      exit   (1);
   }
   strcpy (caucus_x, argv[1]);
#endif

/*=====Section 1: check password. */
   /*** Start with "abcdefg".  Permute by SHIP and CODE. */
   strcpy      (result, "abcdefg");
   lstrpermute (result, argv[3]);
   lstrpermute (result, argv[4]);

   /*** Permute by expiration, mask, and users. */
   expires_in = mask = users = reverse = confmax = itemmax = respmax = 0;
   long8      = 0;
   totalusers = 0;
   for (i=5;   i<argc;   ++i) {
      if (streq (argv[i], "-w"))  sscanf (argv[++i], "%d", &expires_in);
      if (streq (argv[i], "-f"))  sscanf (argv[++i], "%d", &mask);
      if (streq (argv[i], "-u"))  sscanf (argv[++i], "%d", &users);
      if (streq (argv[i], "-c"))  sscanf (argv[++i], "%d", &confmax);
      if (streq (argv[i], "-i"))  sscanf (argv[++i], "%d", &itemmax);
      if (streq (argv[i], "-r"))  sscanf (argv[++i], "%d", &respmax);
      if (streq (argv[i], "-t"))  sscanf (argv[++i], "%d", &totalusers);
      if (streq (argv[i], "-b"))  reverse = 1;
      if (streq (argv[i], "-8"))  long8   = 1;
   }
   lnumpermute (result, expires_in / 4);
   lnumpermute (result, expires_in % 4);
   lnumpermute (result, mask);
   lnumpermute (result, users);
   lnumpermute (result, confmax);
   lnumpermute (result, itemmax);
   lnumpermute (result, respmax);
   lnumpermute (result, totalusers);

   /*** Permute by each sysid. */
   for (i=5;   i<argc;   ++i) {
      if (streq (argv[i], "-s"))   lstrpermute (result, argv[i+1]);
   }


   if (NOT streq (result, argv[2])  &&  NOT streq (result2, argv[2])) {
      printf ("Incorrect password: '%s'.\n", argv[2]);
/*    printf ("Correct password is '%s'.\n", result); */
      exit   (1);
   }


/*=====Section 2: open caucus_x, and find license block. */
#if UNIX | WIN32
   if ( (fd = OPEN (caucus_x)) < 0) {
      printf ("Can't open file '%s'\n", caucus_x);
      exit   (1);
   }
#endif

   address = -1L;
   for (i=3;   i<argc;   ++i)
      if (streq (argv[i], "-a"))
#if UNIX | WIN32
         sscanf (argv[++i], "%d", &address);
#endif

#if UNIX | WIN32
   if (address < 0)  address = searchfile (fd, L(0));
#endif

   if (address < 0)  {
      printf ("Can't find locator.\n");
      exit   (1);
   }


/*=====Section 3: put data into license block. =======*/
   sprintf (license.serial, "%s%s", argv[3], argv[4]);
   strcpy  (license.serial, strplain(license.serial));
   license.maxusers    = REV(PLAIN(users));
   if      (expires_in == 0)  license.expires = 0;
   else if (expires_in == 1)  license.expires = systime() + 1;
   else                       license.expires = 915148800L + expires_in * 
                                                     L(24) * L(3600) * L(7);
   license.expires       = REV(PLAIN(license.expires));
   license.function_mask = REV(PLAIN(mask));
   license.confmax       = REV(PLAIN(confmax));
   license.itemmax       = REV(PLAIN(itemmax));
   license.respmax       = REV(PLAIN(respmax));
   license.totalusers    = REV(PLAIN(totalusers));

/*
   licens8.maxusers[1] = licens8.expires[1] = licens8.function_mask[1] = 0;
   licens8.confmax [1] = licens8.itemmax[1] = licens8.totalusers   [1] = 0;
   licens8.respmax [1] = 0;
   sprintf (licens8.serial, "%s%s", argv[3], argv[4]);
   strcpy  (licens8.serial, strplain(licens8.serial));
   licens8.maxusers[0] = REV(PLAIN(users));
   if (expires_in == 0)  licens8.expires[0] = 0;
   else                  licens8.expires[0] = 915148800L + expires_in * 
                                                    L(7) * L(24) * L(3600);
   licens8.expires[0]       = REV(PLAIN(licens8.expires[0]));
   licens8.function_mask[0] = REV(PLAIN(mask));
   licens8.confmax[0]       = REV(PLAIN(confmax));
   licens8.itemmax[0]       = REV(PLAIN(itemmax));
   licens8.respmax[0]       = REV(PLAIN(respmax));
   licens8.totalusers[0]    = REV(PLAIN(totalusers));
*/

   /*** For each sysid on the command line, add it to license. */
   for (j=0, i=5;   i<argc;   ++i) {
      if (streq (argv[i], "-s")) {
         strcpy    (str, argv[++i]);
         strlower  (str);
         sysidform (str);
         strcpy (license.sysid[j++], strplain(str));
         strcpy (licens8.sysid[j++], strplain(str));
      }
   }
   strcpy (license.sysid[j], "");



/*=====Section 4: write license block to caucus_x===== */
   LSEEK (fd, address, 0);
   if (long8) write (fd, (char *) &licens8, sizeof(licens8));
   else       write (fd, (char *) &license, sizeof(license)); 
   CLOSE (fd);


/*=====Section 5: report license information. ======== */
   if (PLAIN(license.expires) != 0)  sysdate (str, PLAIN(REV(license.expires)));
   else                              strcpy  (str, " (never)");
   printf ("%4d %9s %s\n", PLAIN(REV(license.maxusers)), str,
                         strplain(license.serial));
   printf ("%5d %5d %5d\n", confmax, itemmax, respmax);
   for (i=0;   NOT NULLSTR(license.sysid[i]);   ++i)
      printf ("%s\n", strplain(license.sysid[i]));
}


FUNCTION int4 searchfile (fd, start)
   int   fd;
   int4  start;
{
   char  buf[520];
   int4  addr;
   short i, j;

   for (addr=start;   LSEEK (fd, addr, 0), read (fd, buf, 520) == 520;
        addr += 512) {

      for (i=0;   i<512;   ++i) {
         for (j=0;   j<8;   ++j)   if (license.locator[j] != buf[i+j])   break;
         if  (j >= 8)  return (addr + i);
      }
   }

   return (-1L);
}

FUNCTION put16 (fp, base, str)
   FILE *fp;
   int4  base;
   char  str[];
{
   union {char c[4];   int4 l; }  u;
   int    i;

   for (i=0;   i<16;   i+=4) {
      u.c[0] = str[i];
      u.c[1] = str[i+1];
      u.c[2] = str[i+2];
      u.c[3] = str[i+3];
      fprintf (fp, "deposit 0%x=%09x\n", base+i, u.l);
   }
}

FUNCTION int4 rev4 (x)
   int4  x;
{
   union { int4 x;  char y[4];} a, b;

   a.x = x;
   b.y[0] = a.y[3];
   b.y[1] = a.y[2];
   b.y[2] = a.y[1];
   b.y[3] = a.y[0];

   return (b.x);
}
