
/*** REREG41.   Re-register a Caucus 4.1 license.
/
/    Invoked as:
/       rereg41  caucus  key
/
/    Arguments:
/       caucus   name of caucus executable file (caucus_x, swebd, swebs, etc.).
/       key      activation key (RRRRRosEEEpppppCCCuuu)
/
/    Purpose:
/       REREG41 "re-registers" a caucus program (executable),
/       according to the information coded in the 4.1 activation key.
/
/    How it works:
/       Renames "caucus" to "caucus0".  Copies "caucus0" to "caucus",
/       parses the sections of the activation key, and rewrites the
/       license block in "caucus".
/
/       Writes a single digit code to stdout to report success or
/       failure:
/         0  success
/         1  missing arguments
/         2  bad password
/         3  can't open file
/         4  can't find license block locator in file
/         5  activation key has bad format (not 21 chars)
/         6  bad operating system code
/
/       Based on lice41.c
/
/    Exit status:
/
/    Known bugs:
/      Must add system-type codes for platforms that use reverse-endian
/      math.  (See "reverse-endian" comment.)
/
/      Similar question about 'long8'... DEC-UNIX, perhaps?
/
/    Home:  xlice/rereg41.c
/ */

/*** Copyright (C) 1999 Screen Porch LLC.  All rights reserved.  */

/*: CR  3/28/99 23:27 New program, based on lice41.c. */

#include <stdio.h>
#include <errno.h>
#include "lice41.h"
#include "license.h"
#include "systype.h"

#define MAXSYSID 16

#if UNIX
#define  OPEN(x)       open (x, 2)
#define  CLOSE(x)      close(x)
#define  LSEEK(x,y,z)  lseek (x, y, z)
#endif

#if WIN32
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#define  OPEN(x)       _open (x, O_BINARY | O_RDWR)
#define  CLOSE(x)      _close(x)
#define  LSEEK(x,y,z)  _lseek(x, y, z)
#endif

#define  REV(x)    (reverse ? rev4(x) : x)

static struct license_t license =
   {0x11, 0x22, 0x33, 0x44, 0xAA, 0xBB, 0xCC, 0xDD};

static struct license8_t {
   char   locator[8];
   int4   expires[2];
   int4   maxusers[2];
   int4   function_mask[2];
   int4   confmax[2];
   int4   itemmax[2];
   int4   respmax[2];
   int4   totalusers[2];
   int4   reserve2[2];
   char   serial[16];
   char   sysid[16][16];
} licens8 = {0x11, 0x22, 0x33, 0x44, 0xAA, 0xBB, 0xCC, 0xDD};

FUNCTION  main (argc, argv)
   int   argc;
   char *argv[];
{
   char  caucus_x[200], result[40], backup[200], lowtime[20];
   int4  address, rc;
   int4  mask, expires_in, users, confmax, itemmax, respmax, totalusers;
   int4  searchfile(), systime();
   int   fd, i, j, reverse, long8, weeks, bi;
   char  str[40];
   char  customer[20], os[20], password[20];
   char *strplain(), *strcpy();
   int4  rev4();

   if (argc < 3) {
      printf ("1\n");
      exit   (1);
   }
   if (strlen(argv[2]) != 21) {
      printf ("5\n");
      exit   (1);
   }
   strcpy (caucus_x, argv[1]);

   /*** Parse activation key, and set standard values of other license
   /    parameters. */
   sscanf (argv[2], "%5s%2s%3d%5s%3d%3d",
                    customer, os, &weeks, password, &confmax, &totalusers);
   /*** Which systems use reverse-endian math? */
   reverse    = 0;
#if 0
   reverse    = 1;
#endif
   long8      = 0;
   expires_in = weeks;
   mask       = 5;
   users      = 0;
   totalusers = totalusers * 100;
   itemmax    = 0;
   respmax    = 0;

/*=====Section 1: check password. */
   /*** Start with "abcdefg".  Permute by customer # and OS. */
   strcpy      (result, "abcde");
   lstrpermute (result, customer);
   lstrpermute (result, os);
   lnumpermute (result, customer[4]);

   /*** Permute by other license parameters. */
   lnumpermute (result, expires_in / 4);
   lnumpermute (result, expires_in % 4);
   lnumpermute (result, mask);
   lnumpermute (result, confmax);
   lnumpermute (result, totalusers);
   lnumpermute (result, totalusers + 2);

   if (NOT streq (result, password)) {
      printf ("2\n");
      exit   (1);
   }

   /*** Check operating system. */
#if WIN32
   if (NOT streq (os, "NT"))  { printf ("6\n");   exit (1); }
#endif

#if LNX12
   if (NOT streq (os, "LX"))  { printf ("6\n");   exit (1); }
#endif

#if AIX41
   if (NOT streq (os, "AX"))  { printf ("6\n");   exit (1); }
#endif

#if DEC32
   if (NOT streq (os, "DU"))  { printf ("6\n");   exit (1); }
#endif

#if HPXA9
   if (NOT streq (os, "HP"))  { printf ("6\n");   exit (1); }
#endif

#if IRX53
   if (NOT streq (os, "IX"))  { printf ("6\n");   exit (1); }
#endif

#if SOL25
   if (NOT streq (os, "SO"))  { printf ("6\n");   exit (1); }
#endif

/*** For the moment, this leaves BSD21 and FBS22. */


/*=====Section 2: Prepare backup file and new copy to be reregistered. */
   /*** Change 'backup' "caucus" to "caucus0" (or "caucus0.exe"). */
   strcpy  (backup, caucus_x);
   sprintf (lowtime, "%04d.", systime() % 10000);
#if WIN32
   for (bi=strlen(backup)-1;   bi > 0;   --bi) {
      if (backup[bi] == '.') {
         stralter (backup+bi, ".", lowtime);
         break;
      }
   }
#endif
#if UNIX
   strcat (backup, lowtime);
#endif

   /*** Rename original caucus executable to 'backup' name; then
   /    copy backup to original -- so we can edit the new copy
   /    under the original name. */
   sysunlink (backup);
   sysrename (caucus_x, backup);
   syscopy   (backup,   caucus_x);


/*=====Section 3: open caucus_x, and find license block. */
#if UNIX | WIN32
   if ( (fd = OPEN (caucus_x)) < 0) {
      extern int errno;
      printf ("3, errno=%d\n", errno);
      exit   (1);
   }
#endif

#if UNIX | WIN32
   address = searchfile (fd, L(0));
#endif

   if (address < 0)  {
      printf ("4\n");
      exit   (1);
   }


/*=====Section 3: put data into license block. =======*/
   sprintf (license.serial, "%s%s", customer, os);
   strcpy  (license.serial, strplain(license.serial));
   license.maxusers    = REV(PLAIN(users));
   if      (expires_in == 0)  license.expires = 0;
   else if (expires_in == 1)  license.expires = systime() + 1;
   else                       license.expires = 915148800L + expires_in * 
                                                     L(24) * L(3600) * L(7);
   license.expires       = REV(PLAIN(license.expires));
   license.function_mask = REV(PLAIN(mask));
   license.confmax       = REV(PLAIN(confmax));
   license.itemmax       = REV(PLAIN(itemmax));
   license.respmax       = REV(PLAIN(respmax));
   license.totalusers    = REV(PLAIN(totalusers));

/*
   licens8.maxusers[1] = licens8.expires[1] = licens8.function_mask[1] = 0;
   licens8.confmax [1] = licens8.itemmax[1] = licens8.totalusers   [1] = 0;
   licens8.respmax [1] = 0;
   sprintf (licens8.serial, "%s%s", argv[3], argv[4]);
   strcpy  (licens8.serial, strplain(licens8.serial));
   licens8.maxusers[0] = REV(PLAIN(users));
   if (expires_in == 0)  licens8.expires[0] = 0;
   else                  licens8.expires[0] = 915148800L + expires_in * 
                                                    L(7) * L(24) * L(3600);
   licens8.expires[0]       = REV(PLAIN(licens8.expires[0]));
   licens8.function_mask[0] = REV(PLAIN(mask));
   licens8.confmax[0]       = REV(PLAIN(confmax));
   licens8.itemmax[0]       = REV(PLAIN(itemmax));
   licens8.respmax[0]       = REV(PLAIN(respmax));
   licens8.totalusers[0]    = REV(PLAIN(totalusers));
*/

   strcpy (license.sysid[0], "");



/*=====Section 4: write license block to caucus_x===== */
   rc = LSEEK (fd, address, 0);
   if (long8) write (fd, (char *) &licens8, sizeof(licens8));
   else       rc = write (fd, (char *) &license, sizeof(license)); 
/* else       rc = write (fd, (char *) &license, 56);  */
   rc = CLOSE (fd);
  


/*=====Section 5: report license information. ======== */
   if (PLAIN(license.expires) != 0)  sysdate (str, PLAIN(REV(license.expires)));
   else                              strcpy  (str, " (never)");
/*
   printf ("%4d %9s %s\n", PLAIN(REV(license.maxusers)), str, 
   printf ("%5d %5d %5d\n", confmax, itemmax, respmax);
*/

   printf ("0\n");
}


FUNCTION int4 searchfile (fd, start)
   int   fd;
   int4  start;
{
   char  buf[520];
   int4  addr;
   int   lr, rr;
   short i, j;

   for (addr=start;   (lr = LSEEK (fd, addr, 0)), 
                      (rr = read (fd, buf, 520)) == 520;
        addr += 512) {

/*    printf ("lr=%9d,    rr=%3d\n", lr, rr); */
      for (i=0;   i<512;   ++i) {
         for (j=0;   j<8;   ++j)   if (license.locator[j] != buf[i+j])   break;
         if  (j >= 8)  return (addr + i);
      }
   }

   return (-1L);
}


FUNCTION int4 rev4 (x)
   int4  x;
{
   union { int4 x;  char y[4];} a, b;

   a.x = x;
   b.y[0] = a.y[3];
   b.y[1] = a.y[2];
   b.y[2] = a.y[1];
   b.y[3] = a.y[0];

   return (b.x);
}
