
/*** CPW1.   Caucus Password utility # 1
/
/    Invoked as:
/       cpw1  passfile  [lockflag]
/
/    Arguments:
/       passfile     full path name of password file
/       lockflag     if "-l", do not lock password file (only applies for "v")
/
/    Purpose:
/       Provide several tools for handling the password file for
/       an HTTP access authorization controlled directory.
/
/    How it works:
/       Reads one line from stdin to determine the particular function,
/       either "v" (validate), "c" (change), "a" (add), or "d" (delete).
/
/       Validate reads two more lines from stdin (userid, then password).
/       Succeeds if this is a valid userid/password combination.
/
/       Change reads three more lines from stdin (userid, new password,
/       old password).  If id/old password is valid, changes the password.
/
/       Add reads two lines from stdin (userid, new password), and
/       creates the new entry in the password file.  (If the userid already
/       exists, nothing is done.)
/
/       Delete reads one line from stdin (userid) and deletes that
/       entry.  (Delete must be run from the real userid that owns passfile.)
/
/       WARNING!  Must not be used on /etc/passwd!
/
/       Writes a single-digit result code to stdout.  '0' means success,
/       other codes taken from the BAD_xxx macros below.
/
/       NOTE!  See "OS Dependencies" section below.
/
/    Exit status: 0
/
/    Error Conditions:
/ 
/    Known bugs:
/
/    Home:  cpw1.c
/ */

/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: CR  1/08/96 17:37 New program. */
/*: CR  1/29/97 17:18 Allow 19 chars, "_", ".", +/# lines in passwd file. */
/*: CR  1/29/97 23:00 Add 'delete' option. */
/*: CR  2/24/97 11:40 Add "-l" flag. */

/* --- OS Dependencies: some versions of Unix require special definitions. */
#define  SUNOS4   0        /* Define as 1 for SunOS 4.x */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <pwd.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#define  FUNCTION
#define  NOT         !

#define  BAD_NOARG    1    /* no arguments! */
#define  BAD_EXISTS   2    /* userid already exists (add a user) */
#define  BAD_NOENTRY  3    /* userid does not exist (chg password) */
#define  BAD_OLDPW    4    /* bad old password */
#define  BAD_LENGTH   6    /* userid too int4. */
#define  BAD_CHARS    7    /* bad chars in userid. */
#define  BAD_BUSY     8    /* temp password file busy */
#define  BAD_NOREAD   9    /* can't read  password file */
#define  BAD_NOWRITE 10    /* can't write to temp password file */

#define  MAXTRIES    30

#define  LOGFAIL      0    /* Log failure codes to /tmp/failcode? */

#define ascii_to_bin(c) ((c)>='a'?(c-59):(c)>='A'?((c)-53):(c)-'.')
#define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')

static char user[200];

FUNCTION int main (argc, argv)
   int   argc;
   char *argv[];
{
   char   oldpw[200], newpw[200], temp[200], old[200], func[200];
   char   cryptstr[200], passwd[200], name[200];
   char   line[300];
   time_t tm;
   char   salt[2];
   FILE  *fp, *pass;
   char  *ok;
   int    failcode, is_validate, is_add, is_change, is_del, tries, lockfd;
   int    is_lock;
#if SUNOS4
   char  *crypt();
#endif

   /*** Check arguments. */
   if (argc < 2)                                fail (BAD_NOARG);
   is_lock = 1;
   if (argc == 3  &&  strcmp (argv[2], "-l") == 0)  is_lock = 0;

   umask(022);

   /*** Determine the function to apply.  */
   ok = fgets (func,  200, stdin);   func  [strlen(func) -1] = '\0';
   is_change   = (func[0] == 'c');
   is_validate = (func[0] == 'v');
   is_add      = (func[0] == 'a');
   is_del      = (func[0] == 'd');
   if (NOT (is_change | is_validate | is_add | is_del))  fail (BAD_NOARG);

   /*** Get the relevant data (userid, pw, new pw). */
   ok = fgets (user,  200, stdin);   user  [strlen(user) -1] = '\0';
   ok = fgets (newpw, 200, stdin);   newpw [strlen(newpw)-1] = '\0';
   ok = fgets (oldpw, 200, stdin);   oldpw [strlen(oldpw)-1] = '\0';
   if (is_validate)  strcpy (oldpw, newpw);

   /*** Restrict new ("add") userids to certain conditions. */
   if (is_add  &&  is_outside (user, "abcdefghijklmnopqrstuvwxyz"
                                     "0123456789._")) fail (BAD_CHARS);

   if (is_add  &&  strlen(user) > 19)                 fail (BAD_LENGTH);

   /*** Enforce 'delete' must be owner of passfile. */
   if (is_del)  setuid (getuid());

   /*** Create new encrypted password. */
   time(&tm);
   salt[0] = bin_to_ascii( tm       & 0x3f);
   salt[1] = bin_to_ascii((tm >> 5) & 0x3f);
   strcpy (cryptstr, crypt(newpw, salt));
  
   /*** Determine name of TEMP copy of new password file. */
   sprintf (temp, "%s.TMP", argv[1]);
   sprintf (old,  "%s.OLD", argv[1]);

   /*** Lock the password file appropriately. */
   if ( (lockfd = locker (argv[1], is_change | is_add | is_del, is_lock)) < 0) 
                                                             fail (BAD_BUSY);
  
   /*** Create TEMP, and copy password info to it. */
   if ( (pass = fopen (argv[1], "r")) == NULL)               fail (BAD_NOREAD);
   if ( (is_change | is_add | is_del)  &&
        (fp   = fopen (temp,    "w")) == NULL)               fail (BAD_NOWRITE);
   if (is_add)  failcode = 0;
   else         failcode = BAD_NOENTRY;

   while (get_ent (pass, name, passwd, line)) {

      if (strcmp  (user, name) == 0)  {
         if (is_del)      { failcode = 0;             continue; }
         if (is_add)      { failcode = BAD_EXISTS;    break; }
         if (strncmp (passwd, crypt (oldpw, passwd), 13))
                          { failcode = BAD_OLDPW;     break; }
         if (is_validate) { failcode = 0;             break; }
         strcpy (passwd, cryptstr);
         failcode = 0;
      }

      if (is_change | is_add | is_del)  {
         if ( (line[0] == '#') | (line[0] == '+') ) 
               fprintf (fp, "%s\n",    line);
         else  fprintf (fp, "%s:%s\n", name, passwd);
      }
   }
   fclose (pass);

   if (is_validate)  { unlocker (lockfd);   fail (failcode); }

   /*** Exited loop with a failure condition? */
   if (failcode > 0) {
      fclose   (fp);
      unlink   (temp);
      unlocker (lockfd);
      fail     (failcode);
   }

   /*** Add new user if appropriate. */
   if (is_add)   fprintf (fp, "%s:%s\n", user, cryptstr);
   fclose (fp);

   /*** Back up the original password file (to file.OLD), and
   /    put the new (temp) file in its place. */
   unlink   (old);
   link     (argv[1], old);
   unlink   (argv[1]);
   link     (temp, argv[1]);
   unlink   (temp);
   chmod    (argv[1], 0644);
   unlocker (lockfd);

   fail (0);
   exit (0);   /* Keep lint happy. */
}


FUNCTION fail (code)
   int   code;
{
   FILE   *fp;

   printf  ("%c\n", '0' + code);

#if LOGFAIL
   if ( (fp = fopen ("/tmp/failcode", "a")) != NULL) {
      fprintf (fp, "user='%s', code=%d\n", user, code);
      fclose  (fp);
   }
#endif
  
   exit    (0);
}


FUNCTION  get_ent (pass, name, passwd, line)
   FILE  *pass;
   char  *name, *passwd;
   char  *line;
{
   char  *xn, *xl;

   if (fgets (line, 300, pass) == NULL)   return (0);
   line [strlen(line) - 1] = '\0';

   /*** Pluck out user name. */
   for (xn=name, xl=line;  (*xl) && (*xl != ':');   ++xl)   *xn++ = *xl;
   *xn = '\0';

   /*** Pluck out encrypted password. */
   for (xn=passwd, ++xl;   (*xl) && (*xl != ':');   ++xl)   *xn++ = *xl;
   *xn = '\0';

   return (1);
}


FUNCTION  locker (char *name, int write, int is_lock)
{
   struct flock alock = { 0, SEEK_SET, 0, 1 };
   char   lockname[256];
   int    lockflag, i, fd;
   extern int errno;

   if (NOT write  &&  NOT is_lock)  return (0);

   /*** Open the "lockfile", which is NAME plus a trailing "_l". */
   strcpy (lockname, name);
   strcat (lockname, "_l");
   if ( (fd = open (lockname, 2)) < 0)  fd = creat (lockname, 0700);
   if (fd < 0)  return (-1);

   alock.l_type   = (write ? F_WRLCK : F_RDLCK);
   alock.l_start  = 0;

   for (i=0;   (lockflag = fcntl (fd, F_SETLK, &alock)) < 0;   i++) {
/*    fprintf (stderr, "errno=%d\n", errno);   */
      if (i > MAXTRIES)  { close (fd);   return (-2); }
      sleep (2);
   }

   return (fd);
}



FUNCTION  unlocker (int fd)
{
   struct flock alock = { 0, SEEK_SET, 0, 1 };

   if (fd == 0)  return (1);

   alock.l_type   = F_UNLCK;
   alock.l_start  = 0;
   fcntl  (fd, F_SETLK, &alock);
   close  (fd);
   return (1);
}


FUNCTION  is_outside (char *str, char *list)
{
   short  found;
   char  *p;

   for (  ;   *str;   ++str) {
      for (found=0, p=list;   *p;   ++p) {
         if (*str == *p)  { found = 1;  break; }
      }

      if (NOT found)  return (1);
   }

   return (0);
}
