
/*** SYS_SETUID.  Set real & effective user and group ids.
/
/    rc = sys_setuid (argv, pass, effe_id, real_id, effe_group, real_group);
/   
/    Parameters:
/       int   rc;          (return code)
/       char *argv[];      (command line args to calling program)
/       int   pass;        (which pass?)
/       int   effe_id;     (desired effective userid)
/       int   real_id;     (desired real      userid)
/       int   effe_group;  (desired effective group id)
/       int   real_group;  (desired real      group id)
/
/    Purpose:
/       Set the real and effective userid and group numbers for
/       this process.  Note that the process must be running as
/       superuser (root, id 0) for all possible combinations
/       of real & effective userid & group to work.
/
/    How it works:
/       This is very dependent on the particular implementation of
/       Unix.  There are three possible approaches, each inside
/       its own "#if" code section:
/
/         (a) If the system supports setregid() and setreuid(),
/             use those.
/
/         (b) If not, but it supports setresgid() and setresuid(),
/             use those.
/
/         (c) Otherwise, set the real userid and the real group id,
/             and rerun the program via execvp() (or some other
/             version of execl() if execvp() is not available).
/
/             The idea here is that if the ownership and permission
/             bits of the program are set correctly, then the program
/             will be re-started with the proper real user and group id, 
/             and receive the proper effective user and group ids 
/             automatically.
/
/       Sys_setuid() is actually called twice: once during start-up
/       of the master swebd server (pass 0); and once just after
/       the new dedicated subserver is forked (pass 1).
/
/           If (a) or (b) works, pass 0 doesn't do anything; the id changes
/           occur during pass 1.  Otherwise pass 0 does the id change and 
/           restarts the program, in which case pass 1 is a no-op.
/
/           This scheme lets the master swebd server (in case [a] or [b])
/           start as many children (subservers) as possible (assuming it 
/           was run from root), and only once the children are started 
/           do they revert to their proper ids.
/
/       If the program is not run from superuser, then the attempts
/       to change real & effective userid may fail anyway.  This is
/       not necessarily fatal, but it may (ironically) compromise
/       security.  Sys_setuid() is typically used to set the real
/       user and group ids to "nobody" (or something similar) so
/       that the process will have *only* the effective user and
/       group id access permissions available to it.
/
/    Returns: 0 on success
/            -10 if setreuid()  failed (ditto)                (Sec 1)
/            -20 if setresuid() failed (probably bad real_id) (Sec 2)
/            -30 if setuid()    failed (?)                    (Sec 3)
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  sweb/syssetui.c
/ */

/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: CR 11/13/96 23:30 New function. */
/*: CR  4/15/97 12:41 Integrate with new sys types. */

#include <stdio.h>
#include "chixuse.h"
#include "systype.h"
#include "sweb.h"
#include <errno.h>

#if UNIX | NUT40
#include <unistd.h>
#endif

FUNCTION  int sys_setuid (
   char  *argv[],
   int    pass,
   int    effe_id,
   int    real_id,
   int    effe_group,
   int    real_group )
{

/*** Each #if section has the anticipated "correct" system types
/    from systype.h.  These may be wrong for different versions
/    of a particular operating system (e.g., Solaris 2.5 works
/    as SEC1, but Solaris 2.3 apparently does not.) */

#define SEC1  LNX12 | BSD21 | AIX41 | DEC32 | IRX53 | SOL25 | SUN41
#define SEC2  HPXA9 | SOL24
#define SEC3  0

/*** Note that NT doesn't do anything here, at least not yet. */


#if SEC1
   if (pass     == 0)    return (0);
   if (getuid() == 0)    setreuid (0, 0);
   setregid (real_group, effe_group);
   if (getuid() == 0) {
      if (setreuid (real_id, effe_id) < 0)  {
          setreuid (effe_id, effe_id);
          return (-10);
      }
   }
#endif


#if SEC2
   if (pass     == 0)    return (0);
   if (getuid() == 0)    setresuid (0, 0, 0);
   setresgid (real_group, effe_group, real_group);
   if (getuid() == 0) {
      if (setresuid (real_id, effe_id, real_id) < 0)  {
          setresuid (effe_id, effe_id, effe_id);
          return (-20);
      }
   }
#endif


#if SEC3
/* fprintf (stderr, "\npid=%d, eid=%d, rid=%d, want=%d\n", 
                               getpid(), geteuid(), getuid(), real_id); */
   if (pass == 0  &&  getuid() == 0) {
      seteuid (0);
      setgid  (real_group);
      if (setuid  (real_id) < 0)  return (-30);
      execvp  (argv[0], argv);
   }
#endif


   return (0);
}
